@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2016 Vaclav Slavik
+ *  Copyright (C) 1999-2017 Vaclav Slavik
  *  Copyright (C) 2005 Olivier Sannier
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
@@ -86,7 +86,6 @@ CatalogItemsComparator::CatalogItemsComparator(const Catalog& catalog, const Sor
     switch (m_order.by)
     {
         case SortOrder::By_Source:
-            // TODO: allow non-English source languages too
             m_collator.reset(icu::Collator::createInstance(catalog.GetSourceLanguage().ToIcu(), err));
             break;
 
@@ -119,9 +118,9 @@ bool CatalogItemsComparator::operator()(int i, int j) const
 
     if ( m_order.errorsFirst )
     {
-        if ( a.GetValidity() == CatalogItem::Val_Invalid && b.GetValidity() != CatalogItem::Val_Invalid )
+        if ( a.HasError() && !b.HasError() )
             return true;
-        else if ( a.GetValidity() != CatalogItem::Val_Invalid && b.GetValidity() == CatalogItem::Val_Invalid )
+        else if ( !a.HasError() && b.HasError() )
             return false;
     }
 
