@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2016 Vaclav Slavik
+ *  Copyright (C) 1999-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -40,9 +40,11 @@
 #include <algorithm>
 
 #include "catalog.h"
-#include "digger.h"
+
+#include "configuration.h"
+#include "extractors/extractor.h"
 #include "gexecute.h"
-#include "progressinfo.h"
+#include "qa_checks.h"
 #include "str_helpers.h"
 #include "utility.h"
 #include "version.h"
@@ -54,8 +56,8 @@
 
 // TODO: split into different file
 #if wxUSE_GUI
+    #include "cloud_sync.h" // FIXME - gross, not GUI-related
     #include <wx/msgdlg.h>
-    #include "summarydlg.h"
 #endif
 
 // ----------------------------------------------------------------------
@@ -264,18 +266,7 @@ void Catalog::HeaderData::UpdateDict()
             SetHeader("Last-Translator", Translator + " <" + TranslatorEmail + ">");
     }
 
-    if (TeamEmail.empty())
-    {
-        SetHeader("Language-Team", Team);
-    }
-    else
-    {
-        if (Team.empty())
-            SetHeader("Language-Team", TeamEmail);
-        else
-            SetHeader("Language-Team", Team + " <" + TeamEmail + ">");
-    }
-
+    SetHeader("Language-Team", LanguageTeam);
     SetHeader("MIME-Version", "1.0");
     SetHeader("Content-Type", "text/plain; charset=" + Charset);
     SetHeader("Content-Transfer-Encoding", "8bit");
@@ -372,21 +363,7 @@ void Catalog::HeaderData::ParseDict()
         }
     }
 
-    dummy = GetHeader("Language-Team");
-    if (!dummy.empty())
-    {
-        wxStringTokenizer tkn(dummy, "<>");
-        if (tkn.CountTokens() != 2)
-        {
-            Team = dummy;
-            TeamEmail = wxEmptyString;
-        }
-        else
-        {
-            Team = tkn.GetNextToken().Strip(wxString::trailing);
-            TeamEmail = tkn.GetNextToken();
-        }
-    }
+    LanguageTeam = GetHeader("Language-Team");
 
     wxString ctype = GetHeader("Content-Type");
     int charsetPos = ctype.Find("; charset=");
@@ -397,7 +374,7 @@ void Catalog::HeaderData::ParseDict()
     }
     else
     {
-        Charset = "iso-8859-1";
+        Charset = "ISO-8859-1";
     }
 
     // Parse language information, with backwards compatibility with X-Poedit-*:
@@ -909,7 +886,7 @@ class CharsetInfoFinder : public CatalogParser
 {
     public:
         CharsetInfoFinder(wxTextFile *f)
-                : CatalogParser(f), m_charset("iso-8859-1") {}
+                : CatalogParser(f), m_charset("ISO-8859-1") {}
         wxString GetCharset() const { return m_charset; }
 
     protected:
@@ -935,7 +912,7 @@ class CharsetInfoFinder : public CatalogParser
                 hdr.FromString(mtranslations[0]);
                 m_charset = hdr.Charset;
                 if (m_charset == "CHARSET")
-                    m_charset = "iso-8859-1";
+                    m_charset = "ISO-8859-1";
                 return false; // stop parsing
             }
             return true;
@@ -1168,8 +1145,7 @@ void Catalog::CreateNewHeader()
         dt.SetHeader("Plural-Forms", "nplurals=INTEGER; plural=EXPRESSION;"); // default invalid value
 
     dt.Project = wxEmptyString;
-    dt.Team = wxEmptyString;
-    dt.TeamEmail = wxEmptyString;
+    dt.LanguageTeam = wxEmptyString;
     dt.Charset = "UTF-8";
     dt.Translator = wxConfig::Get()->Read("translator_name", wxEmptyString);
     dt.TranslatorEmail = wxConfig::Get()->Read("translator_email", wxEmptyString);
@@ -1190,10 +1166,8 @@ void Catalog::CreateNewHeader(const Catalog::HeaderData& pot_header)
 
     // clear the fields that are translation-specific:
     dt.Lang = Language();
-    if (dt.Team == "LANGUAGE")
-        dt.Team.clear();
-    if (dt.TeamEmail == "LL@li.org")
-        dt.TeamEmail.clear();
+    if (dt.LanguageTeam == "LANGUAGE <LL@li.org>")
+        dt.LanguageTeam.clear();
 
     // translator should be pre-filled & not the default "FULL NAME <EMAIL@ADDRESS>"
     dt.DeleteHeader("Last-Translator");
@@ -1332,8 +1306,7 @@ void Catalog::FixupCommonIssues()
     if (m_header.GetHeader("Language-Team") == "LANGUAGE <LL@li.org>")
     {
         m_header.DeleteHeader("Language-Team");
-        m_header.Team.clear();
-        m_header.TeamEmail.clear();
+        m_header.LanguageTeam.clear();
     }
 
     if (m_header.GetHeader("Last-Translator") == "FULL NAME <EMAIL@ADDRESS>")
@@ -1591,9 +1564,10 @@ bool Catalog::Save(const wxString& po_file, bool save_mo,
         // msgcat always outputs Unix line endings, so we need to reformat the file
         if (msgcat_ok && outputCrlf == wxTextFileType_Dos)
         {
+            wxCSConv conv(m_header.Charset);
             wxTextFile finalFile(po_file_temp2);
-            if (finalFile.Open())
-                finalFile.Write(outputCrlf);
+            if (finalFile.Open(conv))
+                finalFile.Write(outputCrlf, conv);
         }
 
         if (!TempOutputFileFor::ReplaceFile(po_file_temp2, po_file))
@@ -1625,7 +1599,16 @@ bool Catalog::Save(const wxString& po_file, bool save_mo,
     
     /* If the user wants it, compile .mo file right now: */
 
-    if (m_fileType == Type::PO && save_mo && wxConfig::Get()->Read("compile_mo", (long)true))
+    bool compileMO = save_mo;
+#if wxUSE_GUI // FIXME - gross
+    if (!m_cloudSync || !m_cloudSync->NeedsMO())
+#endif
+    {
+        if (!wxConfig::Get()->Read("compile_mo", (long)true))
+            compileMO = false;
+    }
+
+    if (m_fileType == Type::PO && compileMO)
     {
         const wxString mo_file = wxFileName::StripExtension(po_file) + ".mo";
         TempOutputFileFor mo_file_temp_obj(mo_file);
@@ -2047,9 +2030,9 @@ int Catalog::DoValidate(const wxString& po_file)
     );
 
     for (auto& i: m_items)
-    {
-        i->SetValidity(CatalogItem::Val_Valid);
-    }
+        i->ClearIssue();
+
+    QAChecker::GetFor(*this)->Check(*this);
 
     for ( GettextErrors::const_iterator i = err.begin(); i != err.end(); ++i )
     {
@@ -2058,8 +2041,7 @@ int Catalog::DoValidate(const wxString& po_file)
             auto item = FindItemByLine(i->line);
             if ( item )
             {
-                item->SetValidity(CatalogItem::Val_Invalid);
-                item->SetErrorString(i->text);
+                item->SetIssue(CatalogItem::Issue::Error, i->text);
                 continue;
             }
         }
@@ -2186,105 +2168,74 @@ bool Catalog::HasSourcesAvailable() const
     return true;
 }
 
-#if wxUSE_GUI // TODO: better separation into another file
-bool Catalog::Update(ProgressInfo *progress, bool summary, UpdateResultReason& reason)
+std::shared_ptr<SourceCodeSpec> Catalog::GetSourceCodeSpec() const
 {
-    reason = UpdateResultReason::Unspecified;
-
     if (!m_isOk)
-        return false;
+        return nullptr;
 
-    wxString cwd = wxGetCwd();
     auto path = GetSourcesBasePath();
     if (!path.empty())
     {
         if (!wxFileName::DirExists(path))
-        {
-            reason = UpdateResultReason::NoSourcesFound;
-            return false;
-        }
-
-        wxSetWorkingDirectory(path);
+            return nullptr;
     }
 
-    SourceDigger dig(progress);
+    auto spec = std::make_shared<SourceCodeSpec>();
+    spec->BasePath = !path.empty() ? path : ".";
+    spec->SearchPaths = m_header.SearchPaths;
+    spec->ExcludedPaths = m_header.SearchPathsExcluded;
+    spec->Charset = m_header.SourceCodeCharset;
+    spec->Keywords = m_header.Keywords;
+    for (auto& kv: m_header.GetAllHeaders())
+        spec->XHeaders[kv.Key] = kv.Value;
 
-    auto newcat = dig.Dig(m_header.SearchPaths,
-                          m_header.SearchPathsExcluded,
-                          m_header.Keywords,
-                          m_header.SourceCodeCharset,
-                          reason);
-
-    if (progress->Cancelled())
-        reason = UpdateResultReason::CancelledByUser;
-
-    if (newcat)
-    {
-        bool succ = false;
-        if ( progress )
-            progress->UpdateMessage(_("Merging differences..."));
-
-        bool cancelledByUser = false;
-        if (!summary || ShowMergeSummary(newcat, &cancelledByUser))
-        {
-            switch (m_fileType)
-            {
-                case Type::PO:
-                    succ = Merge(newcat);
-                    break;
-                case Type::POT:
-                    m_items = newcat->m_items;
-                    succ = true;
-                    break;
-            }
-        }
-        if (!succ)
-        {
-            newcat.reset();
-        }
-        if (cancelledByUser)
-            reason = UpdateResultReason::CancelledByUser;
-    }
-
-    wxSetWorkingDirectory(cwd);
-
-    return newcat != nullptr;
+    return spec;
 }
-#endif
 
-bool Catalog::UpdateFromPOT(const wxString& pot_file,
-                            bool summary,
-                            UpdateResultReason& reason,
-                            bool replace_header)
+
+bool Catalog::UpdateFromPOT(const wxString& pot_file, bool replace_header)
 {
-    reason = UpdateResultReason::Unspecified;
-    if (!m_isOk) return false;
-
-    CatalogPtr newcat = std::make_shared<Catalog>(pot_file, CreationFlag_IgnoreTranslations);
-
-    if (!newcat->IsOk())
+    CatalogPtr pot = std::make_shared<Catalog>(pot_file, CreationFlag_IgnoreTranslations);
+    if (!pot->IsOk())
     {
         wxLogError(_("'%s' is not a valid POT file."), pot_file.c_str());
         return false;
     }
 
-    bool cancelledByUser = false;
-    if (!summary || ShowMergeSummary(newcat, &cancelledByUser))
-    {
-        if ( !Merge(newcat) )
-            return false;
-        if ( replace_header )
-            CreateNewHeader(newcat->Header());
-        return true;
-    }
-    else
-    {
-        if (cancelledByUser)
-            reason = UpdateResultReason::CancelledByUser;
-        return false;
-    }
+    return UpdateFromPOT(pot, replace_header);
 }
 
+bool Catalog::UpdateFromPOT(CatalogPtr pot, bool replace_header)
+{
+    switch (m_fileType)
+    {
+        case Type::PO:
+        {
+            if (!Merge(pot))
+                return false;
+            break;
+        }
+        case Type::POT:
+        {
+            m_items = pot->m_items;
+            break;
+        }
+    }
+
+    if (replace_header)
+        CreateNewHeader(pot->Header());
+
+    return true;
+}
+
+CatalogPtr Catalog::CreateFromPOT(const wxString& pot_file)
+{
+    CatalogPtr c = std::make_shared<Catalog>();
+    if (c->UpdateFromPOT(pot_file, /*replace_header=*/true))
+        return c;
+    else
+        return nullptr;
+}
 
 bool Catalog::Merge(const CatalogPtr& refcat)
 {
@@ -2301,8 +2252,8 @@ bool Catalog::Merge(const CatalogPtr& refcat)
     refcat->DoSaveOnly(tmp1, wxTextFileType_Unix);
     DoSaveOnly(tmp2, wxTextFileType_Unix);
 
-    wxString flags("-q --force-po");
-    if (wxConfig::Get()->ReadBool("use_tm_when_updating", false) == false)
+    wxString flags("-q --force-po --previous");
+    if (Config::MergeBehavior() == Merge_None)
     {
         flags += " --no-fuzzy-matching";
     }
@@ -2337,68 +2288,6 @@ bool Catalog::Merge(const CatalogPtr& refcat)
 }
 
 
-static inline wxString ItemMergeSummary(const CatalogItemPtr& item)
-{
-    wxString s = item->GetString();
-    if ( item->HasPlural() )
-        s += "|" + item->GetPluralString();
-    if ( item->HasContext() )
-        s += wxString::Format(" [%s]", item->GetContext());
-
-    return s;
-}
-
-void Catalog::GetMergeSummary(const CatalogPtr& refcat,
-                              wxArrayString& snew, wxArrayString& sobsolete)
-{
-    wxASSERT( snew.empty() );
-    wxASSERT( sobsolete.empty() );
-
-    std::set<wxString> strsThis, strsRef;
-
-    for (auto& i: m_items)
-        strsThis.insert(ItemMergeSummary(i));
-    for (auto& i: refcat->m_items)
-        strsRef.insert(ItemMergeSummary(i));
-
-    for (auto& i: strsThis)
-    {
-        if (strsRef.find(i) == strsRef.end())
-            sobsolete.Add(i);
-    }
-
-    for (auto& i: strsRef)
-    {
-        if (strsThis.find(i) == strsThis.end())
-            snew.Add(i);
-    }
-}
-
-bool Catalog::ShowMergeSummary(const CatalogPtr& refcat, bool *cancelledByUser)
-{
-#if wxUSE_GUI
-    if (cancelledByUser)
-        *cancelledByUser = false;
-    if (wxConfig::Get()->ReadBool("show_summary", false))
-    {
-        wxArrayString snew, sobsolete;
-        GetMergeSummary(refcat, snew, sobsolete);
-        MergeSummaryDialog sdlg;
-        sdlg.TransferTo(snew, sobsolete);
-        bool ok = (sdlg.ShowModal() == wxID_OK);
-        if (cancelledByUser)
-            *cancelledByUser = !ok;
-        return ok;
-    }
-    else
-        return true;
-#else
-    (void)refcat;
-    (void)cancelledByUser;
-    return true;
-#endif
-}
-
 static unsigned GetCountFromPluralFormsHeader(const Catalog::HeaderData& header)
 {
     if ( header.HasHeader("Plural-Forms") )
@@ -2493,7 +2382,7 @@ void Catalog::GetStatistics(int *all, int *fuzzy, int *badtokens,
                 (*fuzzy)++;
             ok = false;
         }
-        if (i->GetValidity() == CatalogItem::Val_Invalid)
+        if (i->HasError())
         {
             if (badtokens)
                 (*badtokens)++;
@@ -2540,6 +2429,18 @@ wxString CatalogItem::GetFlags() const
         return wxEmptyString;
 }
 
+wxString CatalogItem::GetFormatFlag() const
+{
+    auto pos = m_moreFlags.find("-format");
+    if (pos == wxString::npos)
+        return wxString();
+    auto space = m_moreFlags.find_last_of(" \t", pos);
+    if (space == wxString::npos)
+        return m_moreFlags.substr(0, pos);
+    else
+        return m_moreFlags.substr(space+1, pos-space-1);
+}
+
 void CatalogItem::SetFuzzy(bool fuzzy)
 {
     if (!fuzzy && m_isFuzzy)
@@ -2574,7 +2475,7 @@ void CatalogItem::SetTranslation(const wxString &t, unsigned idx)
         m_translations.Add(wxEmptyString);
     m_translations[idx] = t;
 
-    m_validity = Val_Unknown;
+    ClearIssue();
 
     m_isTranslated = true;
     for (size_t i = 0; i < m_translations.GetCount(); i++)
@@ -2591,7 +2492,7 @@ void CatalogItem::SetTranslations(const wxArrayString &t)
 {
     m_translations = t;
 
-    m_validity = Val_Unknown;
+    ClearIssue();
 
     m_isTranslated = true;
     for (size_t i = 0; i < m_translations.GetCount(); i++)
@@ -2606,9 +2507,9 @@ void CatalogItem::SetTranslations(const wxArrayString &t)
 
 void CatalogItem::SetTranslationFromSource()
 {
-    m_validity = Val_Unknown;
+    ClearIssue();
     m_isFuzzy = false;
-    m_isAutomatic = false;
+    m_isPreTranslated = false;
     m_isTranslated = true;
 
     auto iter = m_translations.begin();
@@ -2635,7 +2536,7 @@ void CatalogItem::SetTranslationFromSource()
 void CatalogItem::ClearTranslation()
 {
     m_isFuzzy = false;
-    m_isAutomatic = false;
+    m_isPreTranslated = false;
     m_isTranslated = false;
     for (auto& t: m_translations)
     {
@@ -2694,6 +2595,8 @@ wxString CatalogItem::GetOldMsgid() const
             line.Remove(0, 1);
         if (line.StartsWith("msgid \""))
             line.Remove(0, 7);
+        else if (line.StartsWith("msgid_plural \""))
+            line.replace(0, 14, "\n");
         s += UnescapeCString(line);
     }
     return s;
