@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2016 Vaclav Slavik
+ *  Copyright (C) 1999-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -38,7 +38,8 @@
 #include <memory>
 #include <vector>
 
-class ProgressInfo;
+class CloudSyncDestination;
+struct SourceCodeSpec;
 
 class Catalog;
 class CatalogItem;
@@ -63,13 +64,6 @@ typedef enum
     BOOKMARK_LAST
 } Bookmark;
 
-/// Result of Catalog::Update()
-enum class UpdateResultReason
-{
-    Unspecified,
-    CancelledByUser,
-    NoSourcesFound
-};
 
 /** This class holds information about one particular string.
     This includes source string and its occurrences in source code
@@ -90,8 +84,7 @@ class CatalogItem
                   m_isFuzzy(false),
                   m_isTranslated(false),
                   m_isModified(false),
-                  m_isAutomatic(false),
-                  m_validity(Val_Unknown),
+                  m_isPreTranslated(false),
                   m_lineNum(0),
                   m_bookmark(NO_BOOKMARK) {}
 
@@ -109,13 +102,11 @@ class CatalogItem
                   m_isFuzzy(dt.m_isFuzzy),
                   m_isTranslated(dt.m_isTranslated),
                   m_isModified(dt.m_isModified),
-                  m_isAutomatic(dt.m_isAutomatic),
+                  m_isPreTranslated(dt.m_isPreTranslated),
                   m_hasBadTokens(dt.m_hasBadTokens),
                   m_moreFlags(dt.m_moreFlags),
                   m_comment(dt.m_comment),
-                  m_validity(dt.m_validity),
                   m_lineNum(dt.m_lineNum),
-                  m_errorString(dt.m_errorString),
                   m_bookmark(dt.m_bookmark) {}
 
         /// Gets numeric, 1-based ID
@@ -185,7 +176,7 @@ class CatalogItem
         void SetString(const wxString& s)
         {
             m_string = s;
-            m_validity = Val_Unknown;
+            ClearIssue();
         }
 
         /// Sets the plural form (if applicable).
@@ -231,6 +222,10 @@ class CatalogItem
         /// Gets gettext flags. \see SetFlags
         wxString GetFlags() const;
 
+        /// Returns format flag ("php" for "php-format" etc.) if there's any,
+        // empty string otherwise
+        wxString GetFormatFlag() const;
+
         /// Sets fuzzy flag.
         void SetFuzzy(bool fuzzy);
         /// Gets value of fuzzy flag.
@@ -243,10 +238,10 @@ class CatalogItem
         void SetModified(bool modified) { m_isModified = modified; }
         /// Gets value of modified flag.
         bool IsModified() const { return m_isModified; }
-        /// Sets automatic translation flag.
-        void SetAutomatic(bool automatic) { m_isAutomatic = automatic; }
-        /// Gets value of automatic translation flag.
-        bool IsAutomatic() const { return m_isAutomatic; }
+        /// Sets pre-translated translation flag.
+        void SetPreTranslated(bool pre) { m_isPreTranslated = pre; }
+        /// Gets value of pre-translated translation flag.
+        bool IsPreTranslated() const { return m_isPreTranslated; }
         /// Sets the number of the line this entry occurs on.
         void SetLineNumber(int line) { m_lineNum = line; }
         /// Get line number of this entry.
@@ -273,21 +268,28 @@ class CatalogItem
         wxString GetOldMsgid() const;
         bool HasOldMsgid() const { return !m_oldMsgid.empty(); }
 
-        // Validity (syntax-checking) status of the entry:
-        enum Validity
+        struct Issue
         {
-            Val_Unknown = -1,
-            Val_Invalid = 0,
-            Val_Valid = 1
+            enum Severity
+            {
+                Warning,
+                Error
+            };
+
+            Severity severity;
+            wxString message;
+
+            Issue(Severity s, const wxString& m) : severity(s), message(m) {}
         };
 
-        /** Checks if %i etc. are correct in the translation (true if yes).
-            Strings that are not c-format are always correct. */
-        Validity GetValidity() const { return m_validity; }
-        void SetValidity(Validity val) { m_validity = val; }
+        bool HasIssue() const { return m_issue != nullptr; }
+        bool HasError() const { return m_issue && m_issue->severity == Issue::Error; }
+        const Issue& GetIssue() const { return *m_issue; }
 
-        void SetErrorString(const wxString& str) { m_errorString = str; }
-        wxString GetErrorString() const { return m_errorString; }
+        void ClearIssue() { m_issue.reset(); }
+        void SetIssue(std::shared_ptr<Issue> issue) { m_issue = issue; }
+        void SetIssue(const Issue& issue) { m_issue = std::make_shared<Issue>(issue); }
+        void SetIssue(Issue::Severity severity, const wxString& message) { m_issue = std::make_shared<Issue>(severity, message); }
 
         /// Returns the bookmark for the item
         Bookmark GetBookmark() const {return m_bookmark;}
@@ -309,14 +311,14 @@ class CatalogItem
 
         wxArrayString m_references, m_extractedComments;
         wxArrayString m_oldMsgid;
-        bool m_isFuzzy, m_isTranslated, m_isModified, m_isAutomatic;
+        bool m_isFuzzy, m_isTranslated, m_isModified, m_isPreTranslated;
         bool m_hasBadTokens;
         wxString m_moreFlags;
         wxString m_comment;
-        Validity m_validity;
         int m_lineNum;
-        wxString m_errorString;
         Bookmark m_bookmark;
+
+        std::shared_ptr<Issue> m_issue;
 };
 
 
@@ -492,7 +494,7 @@ class Catalog
 
             wxString Project, CreationDate,
                      RevisionDate, Translator, TranslatorEmail,
-                     Team, TeamEmail, Charset, SourceCodeCharset;
+                     LanguageTeam, Charset, SourceCodeCharset;
             Language Lang;
 
             wxArrayString SearchPaths, SearchPathsExcluded, Keywords;
@@ -552,7 +554,7 @@ class Catalog
             of standard .po format, namely SearchPaths, Keywords, BasePath
             and Language.
 
-            @param flags  CreationFlags combination.
+            @param flags CreationFlags combination.
          */
         bool Load(const wxString& po_file, int flags = 0);
 
@@ -626,18 +628,12 @@ class Catalog
          */
         bool HasSourcesAvailable() const;
 
-        /** Updates the catalog from sources.
-            \see SourceDigger, Parser, UpdateFromPOT.
-         */
-        bool Update(ProgressInfo *progress, bool summary, UpdateResultReason& reason);
+        std::shared_ptr<SourceCodeSpec> GetSourceCodeSpec() const;
 
-        /** Updates the catalog from POT file.
-            \see Update
-         */
-        bool UpdateFromPOT(const wxString& pot_file,
-                           bool summary,
-                           UpdateResultReason& reason,
-                           bool replace_header = false);
+        /// Updates the catalog from POT file.
+        bool UpdateFromPOT(const wxString& pot_file, bool replace_header = false);
+        bool UpdateFromPOT(CatalogPtr pot, bool replace_header = false);
+        static CatalogPtr CreateFromPOT(const wxString& pot_file);
 
         /// Returns the number of strings/translations in the catalog.
         unsigned GetCount() const { return (unsigned)m_items.size(); }
@@ -728,6 +724,9 @@ class Catalog
         /// Returns number of errors (i.e. 0 if no errors).
         int Validate();
 
+        void AttachCloudSync(std::shared_ptr<CloudSyncDestination> c) { m_cloudSync = c; }
+        std::shared_ptr<CloudSyncDestination> GetCloudSync() const { return m_cloudSync; }
+
     protected:
         /// Fix commonly encountered fixable problems with loaded files
         void FixupCommonIssues();
@@ -746,22 +745,6 @@ class Catalog
          */
         bool Merge(const CatalogPtr& refcat);
 
-        /** Returns list of strings that are new in reference catalog
-            (compared to this one) and that are not present in \a refcat
-            (i.e. are obsoleted).
-
-            \see ShowMergeSummary
-         */
-        void GetMergeSummary(const CatalogPtr& refcat,
-                             wxArrayString& snew, wxArrayString& sobsolete);
-
-        /** Shows a dialog with merge summary.
-            \see GetMergeSummary, Merge
-
-            \return true if the merge was OK'ed by the user, false otherwise
-         */
-        bool ShowMergeSummary(const CatalogPtr& refcat, bool *cancelledByUser);
-
     private:
         CatalogItemArray m_items;
         CatalogDeletedDataArray m_deletedItems;
@@ -774,6 +757,8 @@ class Catalog
         wxTextFileType m_fileCRLF;
         int m_fileWrappingWidth;
 
+        std::shared_ptr<CloudSyncDestination> m_cloudSync;
+
         friend class LoadParser;
 };
 
