@@ -1,7 +1,7 @@
 ﻿/*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2015-2016 Vaclav Slavik
+ *  Copyright (C) 2015-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -62,11 +62,12 @@ namespace
 #define OAUTH_URI_PREFIX    "poedit://auth/crowdin/"
 
 // Recursive extract files from /api/project/*/info response
-void ExtractFilesFromInfo(std::vector<std::wstring>& out, const json_dict& r, const std::wstring& prefix)
+void ExtractFilesFromInfo(std::vector<std::wstring>& out, const json& r, const std::wstring& prefix)
 {
-    r.iterate_array("files",[&out,prefix](const json_dict& i){
-        auto name = prefix + i.wstring("name");
-        auto node_type = i.utf8_string("node_type");
+    for (auto i : r["files"])
+    {
+        std::wstring name = prefix + str::to_wstring(i["name"]);
+        std::string node_type = i["node_type"];
         if (node_type == "file")
         {
             out.push_back(name);
@@ -75,7 +76,7 @@ void ExtractFilesFromInfo(std::vector<std::wstring>& out, const json_dict& r, co
         {
             ExtractFilesFromInfo(out, i, name + L"/");
         }
-    });
+    }
 }
 
 } // anonymous namespace
@@ -98,9 +99,9 @@ public:
     {}
 
 protected:
-    std::string parse_json_error(const json_dict& response) const override
+    std::string parse_json_error(const json& response) const override
     {
-        auto msg = response.subdict("error").utf8_string("message");
+        std::string msg = response["error"]["message"];
 
         // Translate commonly encountered messages:
         if (msg == "Translations download is forbidden by project owner")
@@ -131,11 +132,12 @@ CrowdinClient::CrowdinClient() : m_api(new crowdin_http_client(*this))
 CrowdinClient::~CrowdinClient() {}
 
 
-void CrowdinClient::Authenticate(std::function<void()> callback)
+dispatch::future<void> CrowdinClient::Authenticate()
 {
     auto url = WrapLink(OAUTH_AUTHORIZE_URL);
-    m_authCallback = callback;
+    m_authCallback.reset(new dispatch::promise<void>);
     wxLaunchDefaultBrowser(url);
+    return m_authCallback->get_future();
 }
 
 
@@ -151,8 +153,8 @@ void CrowdinClient::HandleOAuthCallback(const std::string& uri)
 
     SaveAndSetToken(m.str(1));
 
-    m_authCallback();
-    m_authCallback = nullptr;
+    m_authCallback->set_value();
+    m_authCallback.reset();
 }
 
 
@@ -162,70 +164,63 @@ bool CrowdinClient::IsOAuthCallback(const std::string& uri)
 }
 
 
-void CrowdinClient::GetUserInfo(std::function<void(UserInfo)> callback)
+dispatch::future<CrowdinClient::UserInfo> CrowdinClient::GetUserInfo()
 {
-    m_api->get("/api/account/profile?json=",
-            // OK:
-            [callback](const json_dict& r){
-                auto profile = r.subdict("profile");
-                UserInfo u;
-                u.name = profile.wstring("name");
-                u.login = profile.wstring("login");
-                callback(u);
-            },
-            // error:
-            [](std::exception_ptr) {}
-    );
+    return m_api->get("/api/account/profile?json=")
+        .then([](json r)
+        {
+            json profile = r["profile"];
+            UserInfo u;
+            u.login = str::to_wstring(profile["login"]);
+            u.name = !profile["name"].is_null() ? str::to_wstring(profile["name"]) : u.login;
+            return u;
+        });
 }
 
 
-void CrowdinClient::GetUserProjects(std::function<void(std::vector<ProjectListing>)> onResult,
-                                    error_func_t onError)
+dispatch::future<std::vector<CrowdinClient::ProjectListing>> CrowdinClient::GetUserProjects()
 {
-    m_api->get("/api/account/get-projects?json=&role=all",
-            // OK:
-            [onResult](const json_dict& r){
-                std::vector<ProjectListing> all;
-                r.iterate_array("projects",[&all](const json_dict& i){
-                    all.push_back({i.wstring("name"),
-                                   i.utf8_string("identifier"),
-                                   (bool)i.number("downloadable")});
+    return m_api->get("/api/account/get-projects?json=&role=all")
+        .then([](json r)
+        {
+            std::vector<ProjectListing> all;
+            for (auto i : r["projects"])
+            {
+                all.push_back({
+                    str::to_wstring(i["name"]),
+                    i["identifier"],
+                    (bool)i["downloadable"].get<int>()
                 });
-                onResult(all);
-            },
-            onError
-    );
+            }
+            return all;
+        });
 }
 
 
-void CrowdinClient::GetProjectInfo(const std::string& project_id,
-                                   std::function<void(ProjectInfo)> onResult,
-                                   error_func_t onError)
+dispatch::future<CrowdinClient::ProjectInfo> CrowdinClient::GetProjectInfo(const std::string& project_id)
 {
     auto url = "/api/project/" + project_id + "/info?json=&project-identifier=" + project_id;
-    m_api->get(url,
-            // OK:
-            [onResult](const json_dict& r){
-                ProjectInfo prj;
-                auto details = r.subdict("details");
-                prj.name = details.wstring("name");
-                prj.identifier = details.utf8_string("identifier");
-                r.iterate_array("languages",[&prj](const json_dict& i){
-                    if (i.number("can_translate") != 0)
-                        prj.languages.push_back(Language::TryParse(i.wstring("code")));
-                });
-                ExtractFilesFromInfo(prj.files, r, L"/");
-                onResult(prj);
-            },
-            onError
-    );
+    return m_api->get(url)
+        .then([](json r){
+            ProjectInfo prj;
+            auto details = r["details"];
+            prj.name = str::to_wstring(details["name"]);
+            prj.identifier = details["identifier"];
+            for (auto i : r["languages"])
+            {
+                if (i["can_translate"].get<int>() != 0)
+                    prj.languages.push_back(Language::TryParse(str::to_wstring(i["code"])));
+            }
+            ExtractFilesFromInfo(prj.files, r, L"/");
+            return prj;
+        });
 }
 
 
-void CrowdinClient::DownloadFile(const std::string& project_id, const std::wstring& file, const Language& lang,
-                                 const std::wstring& output_file,
-                                 std::function<void()> onSuccess,
-                                 error_func_t onError)
+dispatch::future<void> CrowdinClient::DownloadFile(const std::string& project_id,
+                                                   const std::wstring& file,
+                                                   const Language& lang,
+                                                   const std::wstring& output_file)
 {
     // NB: "export_translated_only" means the translation is not filled with the source string
     //     if there's no translation, i.e. what Poedit wants.
@@ -234,14 +229,14 @@ void CrowdinClient::DownloadFile(const std::string& project_id, const std::wstri
                    "&export_translated_only=1"
                    "&language=" + lang.LanguageTag() +
                    "&file=" + http_client::url_encode(file);
-    m_api->download(url, output_file, onSuccess, onError);
+    return m_api->download(url, output_file);
 }
 
 
-void CrowdinClient::UploadFile(const std::string& project_id, const std::wstring& file, const Language& lang,
-                                 const std::string& file_content,
-                                 std::function<void()> onSuccess,
-                                 error_func_t onError)
+dispatch::future<void> CrowdinClient::UploadFile(const std::string& project_id,
+                                                 const std::wstring& file,
+                                                 const Language& lang,
+                                                 const std::string& file_content)
 {
     auto url = "/api/project/" + project_id + "/upload-translation";
 
@@ -252,21 +247,21 @@ void CrowdinClient::UploadFile(const std::string& project_id, const std::wstring
     data.add_value("import_eq_suggestions", "0");
     data.add_file("files[" + str::to_utf8(file) + "]", "upload.po", file_content);
 
-    m_api->post(url, data, onSuccess, onError);
+    return m_api->post(url, data).then([](json){});
 }
 
 
 bool CrowdinClient::IsSignedIn() const
 {
     std::string token;
-    return keytar::GetPassword("Crowdin", &token);
+    return keytar::GetPassword("Crowdin", "", &token);
 }
 
 
 void CrowdinClient::SignInIfAuthorized()
 {
     std::string token;
-    if (keytar::GetPassword("Crowdin", &token))
+    if (keytar::GetPassword("Crowdin", "", &token))
         SetToken(token);
 }
 
@@ -280,14 +275,14 @@ void CrowdinClient::SetToken(const std::string& token)
 void CrowdinClient::SaveAndSetToken(const std::string& token)
 {
     SetToken(token);
-    keytar::AddPassword("Crowdin", token);
+    keytar::AddPassword("Crowdin", "", token);
 }
 
 
 void CrowdinClient::SignOut()
 {
     m_api->set_authorization("");
-    keytar::DeletePassword("Crowdin");
+    keytar::DeletePassword("Crowdin", "");
 }
 
 
