@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2015-2016 Vaclav Slavik
+ *  Copyright (C) 2015-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -28,9 +28,9 @@
 
 #ifdef HAVE_HTTP_CLIENT
 
-#include <functional>
 #include <memory>
 
+#include "concurrency.h"
 #include "language.h"
 
 
@@ -43,7 +43,7 @@ public:
     /// Return singleton instance of the client.
     static CrowdinClient& Get();
 
-    /// Destroys the singleton, must be called (omly) on app shutdown.
+    /// Destroys the singleton, must be called (only) on app shutdown.
     static void CleanUp();
 
     /// Is the user currently signed into Crowdin?
@@ -52,8 +52,6 @@ public:
     /// Wrap relative Crowdin link to absolute URL
     static std::string WrapLink(const std::string& page);
 
-    typedef std::function<void(std::exception_ptr)> error_func_t;
-
     /**
         Authenticate with Crowdin.
         
@@ -61,7 +59,7 @@ public:
         poedit:// URL and call HandleOAuthCallback. @a callback will be
         called after receiving the OAuth token.
      */
-    void Authenticate(std::function<void()> callback);
+    dispatch::future<void> Authenticate();
     void HandleOAuthCallback(const std::string& uri);
     bool IsOAuthCallback(const std::string& uri);
 
@@ -76,7 +74,7 @@ public:
     };
 
     /// Retrieve information about the current user asynchronously
-    void GetUserInfo(std::function<void(UserInfo)> callback);
+    dispatch::future<UserInfo> GetUserInfo();
 
     /// Project listing info
     struct ProjectListing
@@ -87,8 +85,7 @@ public:
     };
 
     /// Retrieve listing of projects accessible to the user
-    void GetUserProjects(std::function<void(std::vector<ProjectListing>)> onResult,
-                         error_func_t onError);
+    dispatch::future<std::vector<ProjectListing>> GetUserProjects();
 
     /// Project detailed information
     struct ProjectInfo
@@ -100,21 +97,19 @@ public:
     };
 
     /// Retrieve listing of projects accessible to the user
-    void GetProjectInfo(const std::string& project_id,
-                        std::function<void(ProjectInfo)> onResult,
-                        error_func_t onError);
+    dispatch::future<ProjectInfo> GetProjectInfo(const std::string& project_id);
 
     /// Asynchronously download specific Crowdin file into @a output_file.
-    void DownloadFile(const std::string& project_id, const std::wstring& file, const Language& lang,
-                      const std::wstring& output_file,
-                      std::function<void()> onSuccess,
-                      error_func_t onError);
+    dispatch::future<void> DownloadFile(const std::string& project_id,
+                                        const std::wstring& file,
+                                        const Language& lang,
+                                        const std::wstring& output_file);
 
     /// Asynchronously upload specific Crowdin file data.
-    void UploadFile(const std::string& project_id, const std::wstring& file, const Language& lang,
-                    const std::string& file_content,
-                    std::function<void()> onSuccess,
-                    error_func_t onError);
+    dispatch::future<void> UploadFile(const std::string& project_id,
+                                      const std::wstring& file,
+                                      const Language& lang,
+                                      const std::string& file_content);
 
 private:
     CrowdinClient();
@@ -127,7 +122,7 @@ private:
     class crowdin_http_client;
     std::unique_ptr<crowdin_http_client> m_api;
 
-    std::function<void()> m_authCallback;
+    std::shared_ptr<dispatch::promise<void>> m_authCallback;
 
     static CrowdinClient *ms_instance;
 };
