@@ -1,7 +1,7 @@
 ﻿/*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2015-2016 Vaclav Slavik
+ *  Copyright (C) 2015-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,7 @@
 #include "crowdin_client.h"
 
 #include "catalog.h"
+#include "cloud_sync.h"
 #include "concurrency.h"
 #include "customcontrols.h"
 #include "errors.h"
@@ -50,6 +51,10 @@
 #include <wx/weakref.h>
 #include <wx/windowptr.h>
 
+#if !wxCHECK_VERSION(3,1,0)
+    #define CenterVertical() Center()
+#endif
+
 #include <boost/algorithm/string.hpp>
 
 CrowdinLoginPanel::CrowdinLoginPanel(wxWindow *parent, int flags)
@@ -88,16 +93,16 @@ CrowdinLoginPanel::CrowdinLoginPanel(wxWindow *parent, int flags)
     sizer->Add(buttons, wxSizerFlags().Expand().Border(wxBOTTOM, 1));
     buttons->Add(learnMore, wxSizerFlags().Center().Border(wxLEFT, PX(LearnMoreLink::EXTRA_INDENT)));
     buttons->AddStretchSpacer();
-    buttons->Add(m_signIn, wxSizerFlags().Right());
-    buttons->Add(m_signOut, wxSizerFlags().Right());
+    buttons->Add(m_signIn, wxSizerFlags());
+    buttons->Add(m_signOut, wxSizerFlags());
 
     if (flags & DialogButtons)
     {
         auto cancel = new wxButton(this, wxID_CANCEL);
 #ifdef __WXMSW__
-        buttons->Add(cancel, wxSizerFlags().Right().Border(wxLEFT, PX(3)));
+        buttons->Add(cancel, wxSizerFlags().Border(wxLEFT, PX(3)));
 #else
-        buttons->Insert(2, cancel, wxSizerFlags().Right().Border(wxRIGHT, PX(6)));
+        buttons->Insert(2, cancel, wxSizerFlags().Border(wxRIGHT, PX(6)));
 #endif
         m_signIn->SetDefault();
         m_signIn->SetFocus();
@@ -165,7 +170,8 @@ void CrowdinLoginPanel::CreateLoginInfoControls(State state)
             name->SetFont(name->GetFont().Bold());
             auto username = new SecondaryLabel(this, m_userLogin);
 
-            sizer->Add(account, wxSizerFlags().PXBorder(wxRIGHT));
+            sizer->Add(account, wxSizerFlags().BORDER_MACOS(wxTOP, PX(3)));
+            sizer->AddSpacer(PX(2));
             auto box = new wxBoxSizer(wxVERTICAL);
             box->Add(name, wxSizerFlags().Left());
             box->Add(username, wxSizerFlags().Left());
@@ -181,19 +187,20 @@ void CrowdinLoginPanel::UpdateUserInfo()
 {
     ChangeState(State::UpdatingInfo);
 
-    CrowdinClient::Get().GetUserInfo(on_main_thread_for_window<CrowdinClient::UserInfo>(
-        this,
-        [=](CrowdinClient::UserInfo u) {
+    CrowdinClient::Get().GetUserInfo()
+        .then_on_window(this, [=](CrowdinClient::UserInfo u) {
             m_userName = u.name;
             m_userLogin = u.login;
             ChangeState(State::SignedIn);
-    }));
+        })
+        .catch_all([](dispatch::exception_ptr){});
 }
 
 void CrowdinLoginPanel::OnSignIn(wxCommandEvent&)
 {
     ChangeState(State::Authenticating);
-    CrowdinClient::Get().Authenticate(on_main_thread(this, &CrowdinLoginPanel::OnUserSignedIn));
+    CrowdinClient::Get().Authenticate()
+        .then_on_window(this, &CrowdinLoginPanel::OnUserSignedIn);
 }
 
 void CrowdinLoginPanel::OnUserSignedIn()
@@ -270,22 +277,22 @@ public:
         topsizer->Add(pickers, wxSizerFlags().Expand().PXDoubleBorderAll());
 
         pickers->Add(new wxStaticText(this, wxID_ANY, _("Project:")),
-                     wxSizerFlags().Center().Right().BORDER_OSX(wxTOP, 1));
+                     wxSizerFlags().CenterVertical().Right().BORDER_MACOS(wxTOP, 1));
         m_project = new wxChoice(this, wxID_ANY);
-        pickers->Add(m_project, wxSizerFlags().Expand().Center());
+        pickers->Add(m_project, wxSizerFlags().Expand().CenterVertical());
 
         pickers->Add(new wxStaticText(this, wxID_ANY, _("Language:")),
-                     wxSizerFlags().Center().Right().BORDER_OSX(wxTOP, 1));
+                     wxSizerFlags().CenterVertical().Right().BORDER_MACOS(wxTOP, 1));
         m_language = new wxChoice(this, wxID_ANY);
-        pickers->Add(m_language, wxSizerFlags().Expand().Center());
+        pickers->Add(m_language, wxSizerFlags().Expand().CenterVertical());
 
         pickers->AddSpacer(PX(5));
         pickers->AddSpacer(PX(5));
 
         pickers->Add(new wxStaticText(this, wxID_ANY, _("File:")),
-                     wxSizerFlags().Center().Right().BORDER_OSX(wxTOP, 1));
+                     wxSizerFlags().CenterVertical().Right().BORDER_MACOS(wxTOP, 1));
         m_file = new wxChoice(this, wxID_ANY);
-        pickers->Add(m_file, wxSizerFlags().Expand().Center());
+        pickers->Add(m_file, wxSizerFlags().Expand().CenterVertical());
 
         m_activity = new ActivityIndicator(this);
         topsizer->AddSpacer(PX(5));
@@ -329,13 +336,12 @@ private:
     void FetchProjects()
     {
         m_activity->Start();
-        CrowdinClient::Get().GetUserProjects(
-            on_main_thread(this, &CrowdinOpenDialog::OnFetchedProjects),
-            m_activity->HandleError
-        );
+        CrowdinClient::Get().GetUserProjects()
+            .then_on_window(this, &CrowdinOpenDialog::OnFetchedProjects)
+            .catch_all(m_activity->HandleError);
     }
 
-    void OnFetchedProjects(const std::vector<CrowdinClient::ProjectListing>& prjs)
+    void OnFetchedProjects(std::vector<CrowdinClient::ProjectListing> prjs)
     {
         m_projects = prjs;
         m_project->Append("");
@@ -362,15 +368,13 @@ private:
         {
             m_activity->Start();
             EnableAllChoices(false);
-            CrowdinClient::Get().GetProjectInfo(
-                m_projects[sel-1].identifier,
-                on_main_thread(this, &CrowdinOpenDialog::OnFetchedProjectInfo),
-                m_activity->HandleError
-            );
+            CrowdinClient::Get().GetProjectInfo(m_projects[sel-1].identifier)
+                .then_on_window(this, &CrowdinOpenDialog::OnFetchedProjectInfo)
+                .catch_all(m_activity->HandleError);
         }
     }
 
-    void OnFetchedProjectInfo(const CrowdinClient::ProjectInfo& prj)
+    void OnFetchedProjectInfo(CrowdinClient::ProjectInfo prj)
     {
         m_info = prj;
         // Put supported files first in the list:
@@ -475,14 +479,14 @@ private:
 
         auto outfile = std::make_shared<TempOutputFileFor>(OutLocalFilename);
         CrowdinClient::Get().DownloadFile(
-            crowdin_prj, crowdin_file, crowdin_lang,
-            outfile->FileName().ToStdWstring(),
-            on_main_thread_for_window<>(this, [=]{
+                crowdin_prj, crowdin_file, crowdin_lang,
+                outfile->FileName().ToStdWstring()
+            )
+            .then_on_window(this, [=]{
                 outfile->Commit();
                 AcceptAndClose();
-            }),
-            m_activity->HandleError
-        );
+            })
+            .catch_all(m_activity->HandleError);
     }
 
     bool IsFileSupported(const wxString& name) const
@@ -524,27 +528,6 @@ private:
     int m_supportedFilesCount;
 };
 
-
-class SyncProgressDialog : public wxDialog
-{
-public:
-    SyncProgressDialog(wxWindow *parent)
-        : wxDialog(parent, wxID_ANY, _("Syncing with Crowdin"), wxDefaultPosition, wxDefaultSize, wxCAPTION | wxSYSTEM_MENU)
-    {
-        auto sizer = new wxBoxSizer(wxVERTICAL);
-        sizer->SetMinSize(PX(300), -1);
-        Activity = new ActivityIndicator(this);
-        sizer->AddStretchSpacer();
-        sizer->Add(Activity, wxSizerFlags().Expand().Border(wxALL, PX(25)));
-        sizer->AddStretchSpacer();
-        SetSizerAndFit(sizer);
-        CenterOnParent();
-    }
-
-    ActivityIndicator *Activity;
-};
-
-
 } // anonymous namespace
 
 
@@ -590,20 +573,22 @@ void CrowdinSyncFile(wxWindow *parent, std::shared_ptr<Catalog> catalog,
                         ? Language::TryParse(header.GetHeader("X-Crowdin-Language").ToStdWstring())
                         : catalog->GetLanguage();
 
-    wxWindowPtr<SyncProgressDialog> dlg(new SyncProgressDialog(parent));
+    wxWindowPtr<CloudSyncProgressWindow> dlg(new CloudSyncProgressWindow(parent));
 
-    auto handle_error = on_main_thread<std::exception_ptr>([=](std::exception_ptr e){
-        dlg->EndModal(wxID_CANCEL);
-        wxWindowPtr<wxMessageDialog> err(new wxMessageDialog
-            (
-                parent,
-                _("Syncing with Crowdin failed."),
-                _("Crowdin error"),
-                wxOK | wxICON_ERROR
-            ));
-        err->SetExtendedMessage(DescribeException(e));
-        err->ShowWindowModalThenDo([err](int){});
-    });
+    auto handle_error = [=](dispatch::exception_ptr e){
+        dispatch::on_main([=]{
+            dlg->EndModal(wxID_CANCEL);
+            wxWindowPtr<wxMessageDialog> err(new wxMessageDialog
+                (
+                    parent,
+                    _("Syncing with Crowdin failed."),
+                    _("Crowdin error"),
+                    wxOK | wxICON_ERROR
+                ));
+            err->SetExtendedMessage(DescribeException(e));
+            err->ShowWindowModalThenDo([err](int){});
+        });
+    };
 
     dlg->Activity->Start(_(L"Uploading translations…"));
 
@@ -611,20 +596,23 @@ void CrowdinSyncFile(wxWindow *parent, std::shared_ptr<Catalog> catalog,
     // This must be done right after entering the modal loop (on non-OSX)
     dlg->CallAfter([=]{
         CrowdinClient::Get().UploadFile(
-            str::to_utf8(crowdin_prj), str::to_wstring(crowdin_file), crowdin_lang,
-            catalog->SaveToBuffer(),
-            [=]{
+                str::to_utf8(crowdin_prj), str::to_wstring(crowdin_file), crowdin_lang,
+                catalog->SaveToBuffer()
+            )
+            .then([=]{
                 auto tmpdir = std::make_shared<TempDirectory>();
                 auto outfile = tmpdir->CreateFileName("crowdin.po");
 
-                call_on_main_thread([=]{
+                dispatch::on_main([=]{
                     dlg->Activity->Start(_(L"Downloading latest translations…"));
                 });
 
-                CrowdinClient::Get().DownloadFile(
-                    str::to_utf8(crowdin_prj), str::to_wstring(crowdin_file), crowdin_lang,
-                    outfile.ToStdWstring(),
-                    on_main_thread([=]{
+                return CrowdinClient::Get().DownloadFile(
+                        str::to_utf8(crowdin_prj), str::to_wstring(crowdin_file), crowdin_lang,
+                        outfile.ToStdWstring()
+                    )
+                    .then_on_main([=]
+                    {
                         CatalogPtr newcat = std::make_shared<Catalog>(outfile);
                         newcat->SetFileName(catalog->GetFileName());
 
@@ -632,11 +620,27 @@ void CrowdinSyncFile(wxWindow *parent, std::shared_ptr<Catalog> catalog,
                         dlg->EndModal(wxID_OK);
 
                         onDone(newcat);
-                    }),
-                    handle_error);
-            },
-            handle_error);
+                    })
+                    .catch_all(handle_error);
+            })
+            .catch_all(handle_error);
     });
 
     dlg->ShowWindowModal();
 }
+
+
+dispatch::future<void> CrowdinSyncDestination::Upload(CatalogPtr file)
+{
+    const auto& header = file->Header();
+    auto crowdin_prj = header.GetHeader("X-Crowdin-Project");
+    auto crowdin_file = header.GetHeader("X-Crowdin-File");
+    auto crowdin_lang = header.HasHeader("X-Crowdin-Language")
+                        ? Language::TryParse(header.GetHeader("X-Crowdin-Language").ToStdWstring())
+                        : file->GetLanguage();
+
+    return CrowdinClient::Get().UploadFile(
+                str::to_utf8(crowdin_prj), str::to_wstring(crowdin_file), crowdin_lang,
+                file->SaveToBuffer()
+            );
+}
