@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2015-2016 Vaclav Slavik
+ *  Copyright (C) 2015-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -49,7 +49,9 @@
 #endif
 
 #include "str_helpers.h"
+#include "unicode_helpers.h"
 
+#include <map>
 #include <memory>
 
 namespace
@@ -59,15 +61,30 @@ wxString WrapTextAtWidth(const wxString& text_, int width, Language lang, wxWind
 {
     if (text_.empty())
         return text_;
+
+#ifdef BIDI_NEEDS_DIRECTION_ON_EACH_LINE
+    wchar_t directionMark = 0;
+    if (bidi::is_direction_mark(*text_.begin()))
+        directionMark = *text_.begin();
+#endif
+        
     auto text = str::to_icu(text_);
 
-    static std::unique_ptr<icu::BreakIterator> iter;
-    if (!iter)
+    static std::map<std::string, std::shared_ptr<icu::BreakIterator>> lang_iters;
+    std::shared_ptr<icu::BreakIterator> iter;
+    auto lang_name = lang.IcuLocaleName();
+    auto li = lang_iters.find(lang_name);
+    if (li == lang_iters.end())
     {
         UErrorCode err = U_ZERO_ERROR;
         iter.reset(icu::BreakIterator::createLineInstance(lang.IsValid() ? lang.ToIcu() : icu::Locale(), err));
         if (!iter)
             iter.reset(icu::BreakIterator::createLineInstance(icu::Locale::getEnglish(), err));
+        lang_iters[lang_name] = iter;
+    }
+    else
+    {
+        iter = li->second;
     }
 
     iter->setText(text);
@@ -97,7 +114,13 @@ wxString WrapTextAtWidth(const wxString& text_, int width, Language lang, wxWind
                 out += previousSubstr;
                 lineStart = previousPos;
             }
+
             out += '\n';
+#ifdef BIDI_NEEDS_DIRECTION_ON_EACH_LINE
+            if (directionMark)
+                out += directionMark;
+#endif
+
             previousSubstr.clear();
         }
         else if (pos > 0 && text[pos-1] == '\n') // forced line feed
@@ -135,20 +158,36 @@ AutoWrappingText::AutoWrappingText(wxWindow *parent, const wxString& label)
     m_text.Replace("\n", " ");
 
     SetInitialSize(wxSize(10,10));
-    Bind(wxEVT_SIZE, &ExplanationLabel::OnSize, this);
+    Bind(wxEVT_SIZE, &AutoWrappingText::OnSize, this);
 }
 
-void AutoWrappingText::SetAlignment(int align)
+void AutoWrappingText::SetLanguage(Language lang)
 {
-    if (GetWindowStyleFlag() & align)
+    m_language = lang;
+    SetAlignment(m_language.Direction());
+}
+
+void AutoWrappingText::SetAlignment(TextDirection dir)
+{
+    // a quirk of wx API: if the current locale is RTL, the meaning of L and R is reversed
+    // for alignments
+    bool isRTL = (dir == TextDirection::RTL);
+    if (GetLayoutDirection() == wxLayout_RightToLeft)
+        isRTL = !isRTL;
+
+    const int align = isRTL ? wxALIGN_RIGHT : wxALIGN_LEFT;
+    if (HasFlag(align))
         return;
     SetWindowStyleFlag(wxST_NO_AUTORESIZE | align);
 }
 
 void AutoWrappingText::SetAndWrapLabel(const wxString& label)
 {
+    m_text = bidi::platform_mark_direction(label);
+    if (!m_language.IsValid())
+        SetAlignment(bidi::get_base_direction(m_text));
+
     wxWindowUpdateLocker lock(this);
-    m_text = label;
     m_wrapWidth = GetSize().x;
     SetLabelText(WrapTextAtWidth(label, m_wrapWidth, m_language, this));
 
@@ -164,6 +203,10 @@ void AutoWrappingText::OnSize(wxSizeEvent& e)
     if (w == m_wrapWidth)
         return;
 
+    // refuse to participate in crazy-small sizes sizing (will be undone anyway):
+    if (w < 50)
+        return;
+
     wxWindowUpdateLocker lock(this);
 
     m_wrapWidth = w;
@@ -283,7 +326,7 @@ bool LearnMoreLinkXmlHandler::CanHandle(wxXmlNode *node)
 }
 
 
-ActivityIndicator::ActivityIndicator(wxWindow *parent)
+ActivityIndicator::ActivityIndicator(wxWindow *parent, int flags)
     : wxWindow(parent, wxID_ANY), m_running(false)
 {
     auto sizer = new wxBoxSizer(wxHORIZONTAL);
@@ -296,14 +339,40 @@ ActivityIndicator::ActivityIndicator(wxWindow *parent)
     m_label->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
 #endif
 
+    if (flags & Centered)
+        sizer->AddStretchSpacer();
     sizer->Add(m_spinner, wxSizerFlags().Center().Border(wxRIGHT, PX(4)));
     sizer->Add(m_label, wxSizerFlags(1).Center());
+    if (flags & Centered)
+        sizer->AddStretchSpacer();
 
-    HandleError = on_main_thread_for_window<std::exception_ptr>(this, [=](std::exception_ptr e){
-        StopWithError(DescribeException(e));
-    });
+    wxWeakRef<ActivityIndicator> self(this);
+    HandleError = [self](dispatch::exception_ptr e){
+        dispatch::on_main([self,e]{
+            if (self)
+                self->StopWithError(DescribeException(e));
+        });
+    };
 }
 
+
+void ActivityIndicator::UpdateLayoutAfterTextChange()
+{
+    m_label->Wrap(GetSize().x);
+
+    Layout();
+
+    if (GetSizer()->IsShown(m_label))
+    {
+        InvalidateBestSize();
+        SetMinSize(wxDefaultSize);
+        SetMinSize(GetBestSize());
+
+        GetParent()->Layout();
+    }
+}
+
+
 void ActivityIndicator::Start(const wxString& msg)
 {
     m_running = true;
@@ -314,7 +383,8 @@ void ActivityIndicator::Start(const wxString& msg)
     auto sizer = GetSizer();
     sizer->Show(m_spinner);
     sizer->Show(m_label, !msg.empty());
-    Layout();
+
+    UpdateLayoutAfterTextChange();
 
     m_spinner->Start();
 }
@@ -329,7 +399,8 @@ void ActivityIndicator::Stop()
     auto sizer = GetSizer();
     sizer->Hide(m_spinner);
     sizer->Hide(m_label);
-    Layout();
+
+    UpdateLayoutAfterTextChange();
 }
 
 void ActivityIndicator::StopWithError(const wxString& msg)
@@ -339,9 +410,11 @@ void ActivityIndicator::StopWithError(const wxString& msg)
     m_spinner->Stop();
     m_label->SetForegroundColour(*wxRED);
     m_label->SetLabel(msg);
+    m_label->SetToolTip(msg);
 
     auto sizer = GetSizer();
     sizer->Hide(m_spinner);
     sizer->Show(m_label);
-    Layout();
+
+    UpdateLayoutAfterTextChange();
 }
