@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2016 Vaclav Slavik
+ *  Copyright (C) 2014-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -26,6 +26,7 @@
 #ifndef Poedit_customcontrols_h
 #define Poedit_customcontrols_h
 
+#include "concurrency.h"
 #include "language.h"
 
 #include <wx/stattext.h>
@@ -69,7 +70,7 @@ protected:
     Language m_language;
 };
 
-/// Like AutoWrappingText, but allows selecting (OS X) or at least copying (Windows)
+/// Like AutoWrappingText, but allows selecting (macOS) or at least copying (Windows)
 /// the text too.
 class SelectableAutoWrappingText : public AutoWrappingText
 {
@@ -81,7 +82,7 @@ public:
 /** 
     Longer, often multiline, explanation label used to provide more information
     about the effects of some less obvious settings. Typeset using smaller font
-    on OS X and grey appearance. Auto-wraps itself to fit surrounding control's
+    on macOS and grey appearance. Auto-wraps itself to fit surrounding control's
     width.
  */
 class ExplanationLabel : public AutoWrappingText
@@ -136,7 +137,12 @@ public:
 class ActivityIndicator : public wxWindow
 {
 public:
-    ActivityIndicator(wxWindow *parent);
+    enum Flags
+    {
+        Centered = 1,
+    };
+
+    ActivityIndicator(wxWindow *parent, int flags = 0);
 
     /// Start indicating, with optional progress label.
     void Start(const wxString& msg = "");
@@ -151,11 +157,13 @@ public:
     bool IsRunning() const { return m_running; }
 
     /// Convenience function for showing error message in the indicator
-    std::function<void(std::exception_ptr)> HandleError;
+    std::function<void(dispatch::exception_ptr)> HandleError;
 
-    bool HasTransparentBackground() override { return true;  }
+    bool HasTransparentBackground() override { return true; }
 
 private:
+    void UpdateLayoutAfterTextChange();
+
     bool m_running;
     wxActivityIndicator *m_spinner;
     wxStaticText *m_label, *m_error;
