@@ -1,7 +1,7 @@
 ﻿/*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2016 Vaclav Slavik
+ *  Copyright (C) 1999-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -46,7 +46,7 @@
 #include <wx/translation.h>
 
 #ifdef __WXOSX__
-#include "osx_helpers.h"
+#include "macos_helpers.h"
 #endif
 
 #ifdef __WXMSW__
@@ -63,12 +63,14 @@
     #error "Unicode build of wxWidgets is required by Poedit"
 #endif
 
+#include "colorscheme.h"
+#include "concurrency.h"
+#include "configuration.h"
 #include "edapp.h"
 #include "edframe.h"
-#include "concurrency.h"
+#include "extractors/extractor_legacy.h"
 #include "manager.h"
 #include "prefsdlg.h"
-#include "extractor.h"
 #include "chooselang.h"
 #include "customcontrols.h"
 #include "gexecute.h"
@@ -84,23 +86,22 @@
 #include "crowdin_client.h"
 
 #ifdef __WXOSX__
-struct PoeditApp::RecentMenuData
+struct PoeditApp::NativeMacAppData
 {
     NSMenu *menu = nullptr;
     NSMenuItem *menuItem = nullptr;
     wxMenuBar *menuBar = nullptr;
-};
+#ifdef USE_SPARKLE
+    NSObject *sparkleDelegate = nullptr;
 #endif
-
-#ifndef DONT_MIGRATE_LEGACY_TM
-extern bool MigrateLegacyTranslationMemory();
+};
 #endif
 
 
 #ifndef __WXOSX__
 
 // IPC for ensuring that only one instance of Poedit runs at a time. This is
-// handled native on OS X and GtkApplication could do it under GTK+3, but wx
+// handled native on macOS and GtkApplication could do it under GTK+3, but wx
 // doesn't support that and we have to implement everything manually for both
 // Windows and GTK+ ports.
 
@@ -256,7 +257,7 @@ IMPLEMENT_APP(PoeditApp);
 PoeditApp::PoeditApp()
 {
 #ifdef __WXOSX__
-    m_recentMenuData.reset(new RecentMenuData);
+    m_nativeMacAppData.reset(new NativeMacAppData);
 #endif
 }
 
@@ -298,7 +299,7 @@ wxString PoeditApp::GetAppBuildNumber() const
 bool PoeditApp::IsBetaVersion() const
 {
     wxString v = GetAppVersion();
-    return  v.Contains("beta") || v.Contains("rc");
+    return v.Contains("beta") || v.Contains("rc");
 }
 
 bool PoeditApp::CheckForBetaUpdates() const
@@ -371,26 +372,6 @@ bool PoeditApp::OnInit()
     if (!wxFileName::DirExists(configDir))
         wxFileName::Mkdir(configDir, wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL);
     wxString configFile = configDir + "/config";
-
-    // Move legacy config locations to XDG compatible ones:
-    if (!wxFileExists(configFile))
-    {
-        wxString oldconfig = wxGetHomeDir() + "/.poedit";
-        if (wxDirExists(oldconfig))
-        {
-            if (wxFileExists(oldconfig + "/config"))
-                wxRenameFile(oldconfig + "/config", configFile);
-            {
-                wxLogNull null;
-                wxRmdir(oldconfig);
-            }
-        }
-        else if (wxFileExists(oldconfig))
-        {
-            // even older dotfile
-            wxRenameFile(oldconfig, configDir + "/config");
-        }
-    }
 #endif // __UNIX__
 
 #if defined(__WXOSX__)
@@ -398,17 +379,10 @@ bool PoeditApp::OnInit()
 #elif defined(__UNIX__)
     #define CFG_FILE configFile
 #else
-    #define CFG_FILE wxEmptyString
+    #define CFG_FILE wxString()
 #endif
 
 #ifdef __WXOSX__
-    // upgrade from the old location of config file:
-    wxString oldcfgfile = wxStandardPaths::Get().GetUserConfigDir() + "/poedit.cfg";
-    if (wxFileExists(oldcfgfile) && !wxFileExists(CFG_FILE))
-    {
-        wxRenameFile(oldcfgfile, CFG_FILE);
-    }
-
     // Remove legacy Sparkle updates folder that used to accumulate broken download
     // files for some users and eat up disk space:
     auto oldsparkle = wxStandardPaths::Get().GetUserDataDir() + "/.Sparkle";
@@ -416,12 +390,12 @@ bool PoeditApp::OnInit()
         wxFileName::Rmdir(oldsparkle, wxPATH_RMDIR_RECURSIVE);
 #endif
 
-    wxConfigBase::Set(
-        new wxConfig(wxEmptyString, wxEmptyString, CFG_FILE, wxEmptyString, 
-                     wxCONFIG_USE_GLOBAL_FILE | wxCONFIG_USE_LOCAL_FILE));
-    wxConfigBase::Get()->SetExpandEnvVars(false);
+    Config::Initialize(CFG_FILE.ToStdWstring());
 
+#ifndef __WXOSX__
     wxImage::AddHandler(new wxPNGHandler);
+#endif
+
 #ifdef __WXMSW__
     wxImage::AddHandler(new wxICOHandler);
 #endif
@@ -431,7 +405,7 @@ bool PoeditApp::OnInit()
 #if defined(__WXOSX__)
     wxXmlResource::Get()->LoadAllFiles(wxStandardPaths::Get().GetResourcesDir());
 #elif defined(__WXMSW__)
-	wxStandardPaths::Get().DontIgnoreAppSubDir();
+    wxStandardPaths::Get().DontIgnoreAppSubDir();
     wxXmlResource::Get()->LoadAllFiles(wxStandardPaths::Get().GetResourcesDir() + "\\Resources");
 #else
     InitXmlResource();
@@ -462,12 +436,6 @@ bool PoeditApp::OnInit()
     FileHistory().Load(*wxConfig::Get());
 #endif
 
-#ifndef DONT_MIGRATE_LEGACY_TM
-    // NB: It's important to do this before TM is used for the first time.
-    if ( !MigrateLegacyTranslationMemory() )
-        return false;
-#endif
-
 #ifdef __WXMSW__
     AssociateFileTypeIfNeeded();
 #endif
@@ -489,18 +457,16 @@ bool PoeditApp::OnInit()
 #endif // !__WXOSX__
 
 #ifdef USE_SPARKLE
-    Sparkle_Initialize(CheckForBetaUpdates());
+    m_nativeMacAppData->sparkleDelegate = Sparkle_Initialize();
 #endif // USE_SPARKLE
 
 #ifdef __WXMSW__
-    wxString appcast = "https://poedit.net/updates/win/appcast";
+    wxString appcast = "https://poedit.net/updates_v2/win/appcast";
     if ( CheckForBetaUpdates() )
     {
         // Beta versions use unstable feed.
-        appcast = "https://poedit.net/updates/win/appcast/beta";
+        appcast = "https://poedit.net/updates_v2/win/appcast/beta";
     }
-    if (!wxPlatformInfo().CheckOSVersion(6,0)) // XP doesn't support SNI
-        appcast.Replace("https://", "http://");
 
     win_sparkle_set_appcast_url(appcast.utf8_str());
     win_sparkle_set_can_shutdown_callback(&PoeditApp::WinSparkle_CanShutdown);
@@ -513,7 +479,7 @@ bool PoeditApp::OnInit()
 
 #ifndef __WXOSX__
     // If we failed to open any window during startup (e.g. because the user
-    // attempted to open MO files), shut the app down. Don't do this on OS X
+    // attempted to open MO files), shut the app down. Don't do this on macOS
     // where a) the initialization is finished after OnInit() and b) apps
     // without windows are OK.
     if (!PoeditFrame::HasAnyWindow())
@@ -537,13 +503,15 @@ int PoeditApp::OnExit()
     // early -- e.g. before wxConfig is destroyed, so they can save changes
     DeletePendingObjects();
 
+    ColorScheme::CleanUp();
+
     TranslationMemory::CleanUp();
 
 #ifdef HAVE_HTTP_CLIENT
     CrowdinClient::CleanUp();
 #endif
 
-    concurrency_queue::cleanup();
+    dispatch::cleanup();
 
 #ifdef USE_SPARKLE
     Sparkle_Cleanup();
@@ -563,7 +531,7 @@ static wxLayoutDirection g_layoutDirection = wxLayout_Default;
 void PoeditApp::SetupLanguage()
 {
 #if defined(__WXMSW__)
-	wxFileTranslationsLoader::AddCatalogLookupPathPrefix(wxStandardPaths::Get().GetResourcesDir() + "\\Translations");
+    wxFileTranslationsLoader::AddCatalogLookupPathPrefix(wxStandardPaths::Get().GetResourcesDir() + "\\Translations");
     wxFileTranslationsLoader::AddCatalogLookupPathPrefix(GetGettextPackagePath() + "/share/locale");
 #elif defined(__WXOSX__)
     wxFileTranslationsLoader::AddCatalogLookupPathPrefix(GetGettextPackagePath() + "/share/locale");
@@ -670,125 +638,10 @@ void PoeditApp::OpenFiles(const wxArrayString& names)
     }
 }
 
-void PoeditApp::SetDefaultExtractors(wxConfigBase *cfg)
-{
-    ExtractorsDB db;
-    bool changed = false;
-    wxString defaultsVersion = cfg->Read("Parsers/DefaultsVersion",
-                                         "1.2.x");
-    db.Read(cfg);
-
-    // Add extractors for languages supported by gettext itself (but only if the
-    // user didn't already add language with this name himself):
-    static struct
-    {
-        char enableByDefault;
-        const char *name;
-        const char *exts;
-    } s_gettextLangs[] = {
-        { 1, "C/C++",      "*.c;*.cpp;*.cc;*.C;*.c++;*.cxx;*.h;*.hpp;*.hxx;*.hh" },
-        { 1, "C#",         "*.cs" },
-        { 1, "EmacsLisp",  "*.el" },
-        { 1, "GSettings",  "*.gschema.xml" },
-        { 1, "Glade",      "*.glade;*.glade2;*.ui" },
-        { 1, "AppData",    "*.appdata.xml" },
-        { 1, "Java",       "*.java" },
-        { 1, "JavaScript", "*.js" },
-        { 1, "Lisp",       "*.lisp" },
-        { 1, "Lua",        "*.lua" },
-        { 1, "ObjectiveC", "*.m" },
-        { 1, "PHP",        "*.php;*.php3;*.php4;*.phtml" },
-        { 1, "Perl",       "*.pl;*.PL;*.pm;*.perl" },
-        { 1, "Python",     "*.py" },
-        { 0, "RST",        "*.rst" },
-        { 1, "Scheme",     "*.scm" },
-        { 0, "Shell",      "*.sh;*.bash" },
-        { 1, "Smalltalk",  "*.st" },
-        { 1, "TCL",        "*.tcl" },
-        { 1, "Vala",       "*.vala" },
-        { 1, "YCP",        "*.ycp" },
-        { 1, "awk",        "*.awk" },
-        { 1, "librep",     "*.jl" },
-        { 0, NULL, NULL }
-    };
-
-    for (size_t i = 0; s_gettextLangs[i].name != NULL; i++)
-    {
-        // if this lang is already registered, don't overwrite it:
-        if (db.FindExtractor(s_gettextLangs[i].name) != -1)
-            continue;
-
-        wxString langflag;
-        if ( wxStrcmp(s_gettextLangs[i].name, "C/C++") == 0 )
-            langflag = " --language=C++";
-        else
-            langflag = wxString(" --language=") + s_gettextLangs[i].name;
-
-        // otherwise add new extractor:
-        Extractor ex;
-        ex.Name = s_gettextLangs[i].name;
-        ex.Enabled = (bool)s_gettextLangs[i].enableByDefault;
-        ex.Extensions = s_gettextLangs[i].exts;
-        ex.Command = wxString("xgettext") + langflag + " --add-comments=TRANSLATORS: --force-po -o %o %C %K %F";
-        ex.KeywordItem = "-k%k";
-        ex.FileItem = "%f";
-        ex.CharsetItem = "--from-code=%c";
-        db.Data.push_back(ex);
-        changed = true;
-    }
-
-    // If upgrading Poedit to 1.2.4, add dxgettext extractor for Delphi:
-#ifdef __WINDOWS__
-    if (defaultsVersion == "1.2.x")
-    {
-        Extractor p;
-        p.Name = "Delphi (dxgettext)";
-        p.Extensions = "*.pas;*.dpr;*.xfm;*.dfm";
-        p.Command = "dxgettext --so %o %F";
-        p.KeywordItem = wxEmptyString;
-        p.FileItem = "%f";
-        db.Data.push_back(p);
-        changed = true;
-    }
-#endif
-
-    // If upgrading Poedit to 1.2.5, update C++ extractor to handle --from-code:
-    if (defaultsVersion == "1.2.x" || defaultsVersion == "1.2.4")
-    {
-        int cpp = db.FindExtractor("C/C++");
-        if (cpp != -1)
-        {
-            if (db.Data[cpp].Command == "xgettext --force-po -o %o %K %F")
-            {
-                db.Data[cpp].Command = "xgettext --force-po -o %o %C %K %F";
-                db.Data[cpp].CharsetItem = "--from-code=%c";
-                changed = true;
-            }
-        }
-    }
-
-    // Poedit 1.5.6 had a breakage, it add --add-comments without space in front of it.
-    // Repair this automatically:
-    for (size_t i = 0; i < db.Data.size(); i++)
-    {
-        wxString& cmd = db.Data[i].Command;
-        if (cmd.Contains("--add-comments=") && !cmd.Contains(" --add-comments="))
-        {
-            cmd.Replace("--add-comments=", " --add-comments=");
-            changed = true;
-        }
-    }
-
-    if (changed)
-    {
-        db.Write(cfg);
-        cfg->Write("Parsers/DefaultsVersion", GetAppVersion());
-    }
-}
 
 void PoeditApp::SetDefaultCfg(wxConfigBase *cfg)
 {
-    SetDefaultExtractors(cfg);
+    LegacyExtractorsDB::RemoveObsoleteExtractors(cfg);
 
     if (cfg->Read("version", wxEmptyString) == GetAppVersion()) return;
 
@@ -896,9 +749,9 @@ void PoeditApp::HandleCustomURI(const wxString& uri)
         return;
 
 #ifdef HAVE_HTTP_CLIENT
-    // nothing to do yet
     if (CrowdinClient::Get().IsOAuthCallback(uri.ToStdString()))
     {
+        wxConfig::Get()->Write("/6p/crowdin_logged_in", true);
         CrowdinClient::Get().HandleOAuthCallback(uri.ToStdString());
         return;
     }
@@ -916,10 +769,6 @@ bool PoeditApp::OnExceptionInMainLoop()
     {
         throw;
     }
-    catch ( std::exception& )
-    {
-        wxLogError(_("Unhandled exception occurred: %s"), DescribeCurrentException());
-    }
 #ifdef __WXOSX__
     catch ( NSException *e )
     {
@@ -928,7 +777,7 @@ bool PoeditApp::OnExceptionInMainLoop()
 #endif
     catch ( ... )
     {
-        wxLogError(_("Unhandled exception occurred."));
+        wxLogError(_("Unhandled exception occurred: %s"), DescribeCurrentException());
     }
 
     return true;
@@ -968,7 +817,7 @@ BEGIN_EVENT_TABLE(PoeditApp, wxApp)
 END_EVENT_TABLE()
 
 
-// OS X and GNOME apps should open new documents in a new window. On Windows,
+// macOS and GNOME apps should open new documents in a new window. On Windows,
 // however, the usual thing to do is to open the new document in the already
 // open window and replace the current document.
 #ifndef __WXMSW__
@@ -999,7 +848,7 @@ void PoeditApp::OnOpen(wxCommandEvent&)
     wxString path = wxConfig::Get()->Read("last_file_path", wxEmptyString);
 
     wxFileDialog dlg(nullptr,
-                     OSX_OR_OTHER("", _("Open catalog")),
+                     MACOS_OR_OTHER("", _("Open catalog")),
                      path,
                      wxEmptyString,
                      Catalog::GetAllTypesFileMask(),
@@ -1054,31 +903,6 @@ void PoeditApp::OnOpenHist(wxCommandEvent& event)
 
 void PoeditApp::OnAbout(wxCommandEvent&)
 {
-#if 0
-    // Forces translation of several strings that are used for about
-    // dialog internally by wx, but are frequently not translate due to
-    // state of wx's translations:
-
-    // TRANSLATORS: This is titlebar of about dialog, "%s" is application name
-    //              ("Poedit" here, but please use "%s")
-    _("About %s");
-    // TRANSLATORS: This is version information in about dialog, "%s" will be
-    //              version number when used
-    _("Version %s");
-
-    // TRANSLATORS: OS X item in app menu
-    _("Services");
-    // TRANSLATORS: OS X item in app menu, %s is replaced with "Poedit"
-    _("Hide %s");
-    // TRANSLATORS: OS X item in app menu
-    _("Hide Others");
-    // TRANSLATORS: OS X item in app menu
-    _("Show All");
-    // TRANSLATORS: OS X item in app menu, %s is replaced with "Poedit"
-    _("Quit %s");
-    _("Preferences...");
-#endif
-
     wxAboutDialogInfo about;
 
 #ifndef __WXOSX__
@@ -1086,7 +910,7 @@ void PoeditApp::OnAbout(wxCommandEvent&)
     about.SetVersion(wxGetApp().GetAppVersion());
     about.SetDescription(_("Poedit is an easy to use translations editor."));
 #endif
-    about.SetCopyright(L"Copyright \u00a9 1999-2016 Václav Slavík");
+    about.SetCopyright(L"Copyright \u00a9 1999-2017 Václav Slavík");
 #ifdef __WXGTK__ // other ports would show non-native about dlg
     about.SetWebSite("https://poedit.net");
 #endif
@@ -1114,7 +938,7 @@ void PoeditApp::OnQuit(wxCommandEvent&)
     }
 
     // The Close() calls below may not terminate immediately, they may ask for
-    // confirmation window-modally on OS X. So change the behavior to terminate
+    // confirmation window-modally on macOS. So change the behavior to terminate
     // the app when the last window is closed now, instead of calling
     // ExitMainLoop(). This will terminate the app automagically when all the
     // windows are closed.
@@ -1220,11 +1044,14 @@ void PoeditApp::TweakOSXMenuBar(wxMenuBar *bar)
 
     NSMenu *editNS = edit->GetHMenu();
 
+    #pragma clang diagnostic push
+    #pragma clang diagnostic ignored "-Wundeclared-selector"
     AddNativeItem(editNS, 0, _("Undo"), @selector(undo:), @"z");
     AddNativeItem(editNS, 1, _("Redo"), @selector(redo:), @"Z");
+    #pragma clang diagnostic pop
     [editNS insertItem:[NSMenuItem separatorItem] atIndex:2];
     if (pasteItem != -1) pasteItem += 3;
-    if (findItem != -1)  findItem += 3;
+    if (findItem != -1) findItem += 3;
 
     NSMenuItem *item;
     if (pasteItem != -1)
@@ -1303,27 +1130,30 @@ void PoeditApp::CreateFakeOpenRecentMenu()
     NSMenu *mainMenu = [NSApp mainMenu];
  
     NSMenuItem *item = [mainMenu addItemWithTitle:@"File" action:NULL keyEquivalent:@""];
-	NSMenu *menu = [[NSMenu alloc] initWithTitle:NSLocalizedString(@"File", nil)];
+    NSMenu *menu = [[NSMenu alloc] initWithTitle:NSLocalizedString(@"File", nil)];
  
-	item = [menu addItemWithTitle:NSLocalizedString(@"Open Recent", nil)
-						   action:NULL
-					keyEquivalent:@""];
-	NSMenu *openRecentMenu = [[NSMenu alloc] initWithTitle:@"Open Recent"];
-	[openRecentMenu performSelector:@selector(_setMenuName:) withObject:@"NSRecentDocumentsMenu"];
-	[menu setSubmenu:openRecentMenu forItem:item];
-    m_recentMenuData->menuItem = item;
-    m_recentMenuData->menu = openRecentMenu;
+    item = [menu addItemWithTitle:NSLocalizedString(@"Open Recent", nil)
+            action:NULL
+            keyEquivalent:@""];
+    NSMenu *openRecentMenu = [[NSMenu alloc] initWithTitle:@"Open Recent"];
+    #pragma clang diagnostic push
+    #pragma clang diagnostic ignored "-Wundeclared-selector"
+    [openRecentMenu performSelector:@selector(_setMenuName:) withObject:@"NSRecentDocumentsMenu"];
+    #pragma clang diagnostic pop
+    [menu setSubmenu:openRecentMenu forItem:item];
+    m_nativeMacAppData->menuItem = item;
+    m_nativeMacAppData->menu = openRecentMenu;
  
-	item = [openRecentMenu addItemWithTitle:NSLocalizedString(@"Clear Menu", nil)
-									 action:@selector(clearRecentDocuments:)
-							  keyEquivalent:@""];
+    item = [openRecentMenu addItemWithTitle:NSLocalizedString(@"Clear Menu", nil)
+            action:@selector(clearRecentDocuments:)
+            keyEquivalent:@""];
 }
 
 void PoeditApp::InstallOpenRecentMenu(wxMenuBar *bar)
 {
-    if (m_recentMenuData->menuItem)
-        [m_recentMenuData->menuItem setSubmenu:nil];
-    m_recentMenuData->menuBar = nullptr;
+    if (m_nativeMacAppData->menuItem)
+        [m_nativeMacAppData->menuItem setSubmenu:nil];
+    m_nativeMacAppData->menuBar = nullptr;
 
     if (!bar)
         return;
@@ -1338,16 +1168,16 @@ void PoeditApp::InstallOpenRecentMenu(wxMenuBar *bar)
     if (!nativeItem)
         return;
 
-    [nativeItem setSubmenu:m_recentMenuData->menu];
-    m_recentMenuData->menuItem = nativeItem;
-    m_recentMenuData->menuBar = bar;
+    [nativeItem setSubmenu:m_nativeMacAppData->menu];
+    m_nativeMacAppData->menuItem = nativeItem;
+    m_nativeMacAppData->menuBar = bar;
 }
 
 void PoeditApp::OnIdleInstallOpenRecentMenu(wxIdleEvent& event)
 {
     event.Skip();
     auto installed = wxMenuBar::MacGetInstalledMenuBar();
-    if (m_recentMenuData->menuBar != installed)
+    if (m_nativeMacAppData->menuBar != installed)
         InstallOpenRecentMenu(installed);
 }
 
@@ -1356,7 +1186,7 @@ void PoeditApp::OSXOnWillFinishLaunching()
     wxApp::OSXOnWillFinishLaunching();
     CreateFakeOpenRecentMenu();
     // We already create the menu item, this would cause duplicates "thanks" to the weird
-    // way wx's menubar works on OS X:
+    // way wx's menubar works on macOS:
     [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"NSFullScreenMenuItemEverywhere"];
 }
 
@@ -1395,19 +1225,15 @@ void PoeditApp::AssociateFileTypeIfNeeded()
         return; // already associated, nothing to do
 
     auto poCmd = wxString::Format("\"%s\" \"%%1\"", wxStandardPaths::Get().GetExecutablePath());
-    auto poIcon = wxStandardPaths::Get().GetResourcesDir() + "\\Resources\\poedit-translation-generic.ico";
     wxRegKey key1(wxRegKey::HKCU, "Software\\Classes\\.po");
     key1.Create();
     key1.SetValue("", "Poedit.PO");
     wxRegKey key2(wxRegKey::HKCU, "Software\\Classes\\Poedit.PO");
     key2.Create();
-    key2.SetValue("", _("PO Translation"));
+    key2.SetValue("", /*TRANSLATORS:File kind displayed in Finder/Explorer*/_("PO Translation"));
     wxRegKey key3(wxRegKey::HKCU, "Software\\Classes\\Poedit.PO\\Shell\\Open\\Command");
     key3.Create();
     key3.SetValue("", poCmd);
-    wxRegKey key4(wxRegKey::HKCU, "Software\\Classes\\Poedit.PO\\DefaultIcon");
-    key4.Create();
-    key4.SetValue("", poIcon);
 }
 
 
