@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2016 Vaclav Slavik
+ *  Copyright (C) 1999-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -43,6 +43,7 @@
 #include "customcontrols.h"
 #include "hidpi.h"
 #include "utility.h"
+#include "unicode_helpers.h"
 
 namespace
 {
@@ -120,7 +121,7 @@ FileViewer::FileViewer(wxWindow*)
 
 #ifdef __WXOSX__
     wxAcceleratorEntry entries[] = {
-        { wxACCEL_CMD,  'W', wxID_CLOSE }
+        { wxACCEL_CMD, 'W', wxID_CLOSE }
     };
     wxAcceleratorTable accel(WXSIZEOF(entries), entries);
     SetAcceleratorTable(accel);
@@ -299,7 +300,7 @@ wxFileName FileViewer::GetFilename(wxString ref) const
             filename.MakeAbsolute(basePath);
             if ( filename.FileExists() )
             {
-                break; // good, found the file
+                return filename; // good, found the file
             }
             else
             {
@@ -313,14 +314,18 @@ wxFileName FileViewer::GetFilename(wxString ref) const
         }
     }
 
-    return filename;
+    return wxFileName(); // invalid
 }
 
 
 void FileViewer::ShowReferences(CatalogPtr catalog, CatalogItemPtr item, int defaultReference)
 {
     m_basePath = catalog->GetSourcesBasePath();
-    m_references = item->GetReferences();
+    if (m_basePath.empty())
+        m_basePath = wxPathOnly(catalog->GetFileName());
+
+    if (item)
+        m_references = item->GetReferences();
 
     m_file->Clear();
     m_file->Enable(m_references.size() > 1);
@@ -332,7 +337,7 @@ void FileViewer::ShowReferences(CatalogPtr catalog, CatalogItemPtr item, int def
     else
     {
         for (auto& r: m_references)
-            m_file->Append(r);
+            m_file->Append(bidi::platform_mark_direction(r));
         m_file->SetSelection(defaultReference);
         SelectReference(m_references[defaultReference]);
     }
@@ -341,6 +346,13 @@ void FileViewer::ShowReferences(CatalogPtr catalog, CatalogItemPtr item, int def
 void FileViewer::SelectReference(const wxString& ref)
 {
     const wxFileName filename = GetFilename(ref);
+    if (!filename.IsOk())
+    {
+        ShowError(wxString::Format(_("Error opening file %s!"), filename.GetFullPath()));
+        m_openInEditor->Disable();
+        return;
+    }
+
     wxFFile file;
     wxString data;
 
@@ -396,5 +408,7 @@ void FileViewer::OnChoice(wxCommandEvent &event)
 
 void FileViewer::OnEditFile(wxCommandEvent&)
 {
-    wxLaunchDefaultApplication(GetFilename(m_file->GetStringSelection()).GetFullPath());
+    wxFileName filename = GetFilename(bidi::strip_control_chars(m_file->GetStringSelection()));
+    if (filename.IsOk())
+        wxLaunchDefaultApplication(filename.GetFullPath());
 }
