@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2000-2016 Vaclav Slavik
+ *  Copyright (C) 2000-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -33,6 +33,8 @@
 #include <wx/translation.h>
 #include <wx/filename.h>
 
+#include <boost/throw_exception.hpp>
+
 #include "gexecute.h"
 #include "errors.h"
 
@@ -128,7 +130,7 @@ long DoExecuteGettext(const wxString& cmdline_, wxArrayString& gstderr)
 
 #ifdef __WXOSX__
     // Hack alert! On Windows, relocation works, but building with it is too
-    // messy/broken on OS X, so just use some custom hacks instead:
+    // messy/broken on macOS, so just use some custom hacks instead:
     auto sharedir = GetGettextPackagePath() + "/share";
     env.env["POEDIT_LOCALEDIR"] = sharedir + "/locale";
     env.env["GETTEXTDATADIR"] = sharedir + "/gettext";
@@ -139,14 +141,16 @@ long DoExecuteGettext(const wxString& cmdline_, wxArrayString& gstderr)
     wxScopedPtr<wxProcess> process(new wxProcess);
     process->Redirect();
 
-    long retcode = wxExecute(cmdline, wxEXEC_BLOCK, process.get(), &env);
+    long retcode = wxExecute(cmdline, wxEXEC_BLOCK | wxEXEC_NODISABLE | wxEXEC_NOEVENTS, process.get(), &env);
 
 	wxInputStream *std_err = process->GetErrorStream();
     if ( std_err && !ReadOutput(*std_err, gstderr) )
         retcode = -1;
 
     if ( retcode == -1 )
-        throw Exception(wxString::Format(_("Cannot execute program: %s"), cmdline.c_str()));
+    {
+        BOOST_THROW_EXCEPTION(Exception(wxString::Format(_("Cannot execute program: %s"), cmdline.c_str())));
+    }
 
     return retcode;
 }
@@ -159,13 +163,29 @@ bool ExecuteGettext(const wxString& cmdline)
     wxArrayString gstderr;
     long retcode = DoExecuteGettext(cmdline, gstderr);
 
-    for ( size_t i = 0; i < gstderr.size(); i++ )
+    wxString pending;
+    for (auto& ln: gstderr)
     {
-        if ( gstderr[i].empty() )
+        if (ln.empty())
             continue;
-        wxLogError("%s", gstderr[i].c_str());
+
+        // special handling of multiline errors
+        if (ln[0] == ' ' || ln[0] == '\t')
+        {
+            pending += "\n\t" + ln.Strip(wxString::both);
+        }
+        else
+        {
+            if (!pending.empty())
+                wxLogError("%s", pending);
+
+            pending = ln;
+        }
     }
 
+    if (!pending.empty())
+        wxLogError("%s", pending);
+
     return retcode == 0;
 }
 
