@@ -1,7 +1,7 @@
 ﻿/*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2015-2016 Vaclav Slavik
+ *  Copyright (C) 2015-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -40,8 +40,11 @@ void InitHiDPIHandling()
     g_pxScalingFactor = dpi.y / 96.0;
 }
 
+#endif // NEEDS_MANUAL_HIDPI
+
 namespace
 {
+
 void LoadPNGImage(wxImage& img, const wxString& filename)
 {
     img.LoadFile(filename, wxBITMAP_TYPE_PNG);
@@ -56,26 +59,24 @@ void LoadPNGImage(wxImage& img, const wxString& filename)
 
 } // anonymous namespace
 
-#endif // NEEDS_MANUAL_HIDPI
-
-
-wxBitmap LoadScaledBitmap(const wxString& name)
+wxImage LoadScaledBitmap(const wxString& name)
 {
     const wxString filename(name + ".png");
     if (!wxFileExists(filename))
-        return wxNullBitmap;
+        return wxNullImage;
+
+    wxImage img;
 
 #ifdef NEEDS_MANUAL_HIDPI
-    if (HiDPIScalingFactor() > 1.0)
+    if (HiDPIScalingFactor() > 1.25)
     {
-        wxImage img;
         double imgScale = HiDPIScalingFactor();
         const wxString filename_2x(name + "@2x.png");
         if (wxFileExists(filename_2x))
         {
             LoadPNGImage(img, filename_2x);
             if (HiDPIScalingFactor() == 2.0)
-                return wxBitmap(img);
+                return img;
             else
                 imgScale /= 2.0;
         }
@@ -84,6 +85,9 @@ wxBitmap LoadScaledBitmap(const wxString& name)
             LoadPNGImage(img, filename);
         }
 
+        if (!img.IsOk())
+            return wxNullImage;
+
         wxImageResizeQuality quality;
         if (imgScale == 2.0)
             quality = wxIMAGE_QUALITY_NEAREST;
@@ -92,10 +96,11 @@ wxBitmap LoadScaledBitmap(const wxString& name)
         else
             quality = wxIMAGE_QUALITY_BICUBIC;
         img.Rescale(img.GetWidth() * imgScale, img.GetHeight() * imgScale, quality);
-        return wxBitmap(img);
+        return img;
     }
     // else: load normally
 #endif
 
-    return wxBitmap(wxImage(filename, wxBITMAP_TYPE_PNG));
+    LoadPNGImage(img, filename);
+    return img;
 }
