@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2016 Vaclav Slavik
+ *  Copyright (C) 2014-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -36,6 +36,8 @@
 #include <boost/iostreams/filter/gzip.hpp>
 #include <boost/iostreams/filtering_stream.hpp>
 
+#include <boost/throw_exception.hpp>
+
 #include <cpprest/asyncrt_utils.h>
 #include <cpprest/http_client.h>
 #include <cpprest/http_msg.h>
@@ -43,16 +45,19 @@
 
 #ifdef _WIN32
     #include <windows.h>
-    #include <wininet.h>
+    #include <winhttp.h>
     #include <netlistmgr.h>
     #pragma comment(lib, "ole32.lib")
-#endif
 
-using namespace web;
+    // can't include both winhttp.h and wininet.h, so put a declaration here
+    //#include <wininet.h>
+    EXTERN_C DECLSPEC_IMPORT BOOL STDAPICALLTYPE InternetGetConnectedState(__out LPDWORD lpdwFlags, __reserved DWORD dwReserved);
+#endif
 
 namespace
 {
 
+using namespace web;
 using utility::string_t;
 using utility::conversions::to_string_t;
 
@@ -99,72 +104,19 @@ public:
 
 } // anonymous namespace
 
-struct json_dict::native
-{
-    native(const web::json::value& x) : val(x) {}
-
-    web::json::value val;
-
-    const web::json::value& get(const std::string& key)
-    {
-        return val.at(to_string_t(key));
-    }
-};
-
-static inline json_dict make_json_dict(const web::json::value& x)
-{
-    return std::make_shared<json_dict::native>(x);
-}
-
-bool json_dict::is_null(const char *name) const
-{
-    return m_native->get(name).is_null();
-}
-
-json_dict json_dict::subdict(const char *name) const
-{
-    return make_json_dict(m_native->get(name));
-}
-
-std::string json_dict::utf8_string(const char *name) const
-{
-    return str::to_utf8(m_native->get(name).as_string());
-}
-
-std::wstring json_dict::wstring(const char *name) const
-{
-    return str::to_wstring(m_native->get(name).as_string());
-}
-
-int json_dict::number(const char *name) const
-{
-    return m_native->get(name).as_integer();
-}
-
-double json_dict::double_number(const char *name) const
-{
-    return m_native->get(name).as_double();
-}
-
-void json_dict::iterate_array(const char *name, std::function<void(const json_dict&)> on_item) const
-{
-    auto& val = m_native->get(name);
-    if (!val.is_array())
-        return;
-    auto size = val.size();
-    for (size_t i = 0; i < size; ++i)
-    {
-        on_item(make_json_dict(val.at(i)));
-    }
-}
-
-
 
 class http_client::impl
 {
 public:
     impl(http_client& owner, const std::string& url_prefix, int flags)
-        : m_owner(owner), m_native(sanitize_url(url_prefix, flags))
+        : m_owner(owner),
+          m_native
+          (
+              sanitize_url(url_prefix, flags)
+            #ifdef _WIN32
+              , get_client_config()
+            #endif
+          )
     {
         #define make_wide_str(x) make_wide_str_(x)
         #define make_wide_str_(x) L ## x
@@ -205,7 +157,7 @@ public:
             if (SUCCEEDED(hr))
                 return result & (NLM_CONNECTIVITY_IPV4_INTERNET|NLM_CONNECTIVITY_IPV6_INTERNET);
         }
-        // XP or manager fallback (IPv6 ignorant):
+        // manager fallback (IPv6 ignorant):
         DWORD flags;
         return ::InternetGetConnectedState(&flags, 0);
     #else
@@ -218,8 +170,7 @@ public:
         m_auth = std::wstring(auth.begin(), auth.end());
     }
 
-    void get(const std::string& url,
-             std::function<void(const http_response&)> handler)
+    dispatch::future<::json> get(const std::string& url)
     {
         http::http_request req(http::methods::GET);
         req.headers().add(http::header_names::accept,     L"application/json");
@@ -228,27 +179,21 @@ public:
             req.headers().add(http::header_names::authorization, m_auth);
         req.set_request_uri(to_string_t(url));
 
+        return
         m_native.request(req)
         .then([=](http::http_response response)
         {
             handle_error(response);
-            handler(make_json_dict(response.extract_json().get()));
-        })
-        .then([=](pplx::task<void> t)
-        {
-            try { t.get(); }
-            catch (...)
-            {
-                handler(std::current_exception());
-            }
+            return ::json::parse(response.extract_utf8string().get());
         });
     }
 
-    void download(const std::string& url, const std::wstring& output_file, response_func_t handler)
+    dispatch::future<void> download(const std::string& url, const std::wstring& output_file)
     {
         using namespace concurrency::streams;
         auto fileStream = std::make_shared<ostream>();
 
+        return
         fstream::open_ostream(to_string_t(output_file)).then([=](ostream outFile)
         {
             *fileStream = outFile;
@@ -267,22 +212,10 @@ public:
         .then([=](size_t)
         {
             return fileStream->close();
-        })
-        .then([=](pplx::task<void> t)
-        {
-            try
-            {
-                t.get();
-                handler(json_dict());
-            }
-            catch (...)
-            {
-                handler(std::current_exception());
-            }
         });
     }
 
-    void post(const std::string& url, const http_body_data& data, response_func_t handler)
+    dispatch::future<::json> post(const std::string& url, const http_body_data& data)
     {
         http::http_request req(http::methods::POST);
         req.headers().add(http::header_names::accept,     L"application/json");
@@ -295,19 +228,12 @@ public:
         req.set_body(body, data.content_type());
         req.headers().set_content_length(body.size());
 
+        return
         m_native.request(req)
         .then([=](http::http_response response)
         {
             handle_error(response);
-            handler(make_json_dict(response.extract_json().get()));
-        })
-        .then([=](pplx::task<void> t)
-        {
-            try { t.get(); }
-            catch (...)
-            {
-                handler(std::current_exception());
-            }
+            return ::json::parse(response.extract_utf8string().get());
         });
     }
 
@@ -315,7 +241,7 @@ private:
     // handle non-OK responses:
     void handle_error(http::http_response r)
     {
-        if (r.status_code() == http::status_codes::OK)
+        if (r.status_code() >= 200 && r.status_code() < 300)
             return; // not an error
 
         int status_code = r.status_code();
@@ -324,7 +250,7 @@ private:
         {
             try
             {
-                auto json = make_json_dict(r.extract_json().get());
+                auto json = ::json::parse(r.extract_utf8string().get());
                 msg = m_owner.parse_json_error(json);
             }
             catch (...) {} // report original error if parsing broken
@@ -332,47 +258,53 @@ private:
         if (msg.empty())
             msg = str::to_utf8(r.reason_phrase());
         m_owner.on_error_response(status_code, msg);
-        throw http::http_exception(status_code, msg);
+        BOOST_THROW_EXCEPTION(http::http_exception(status_code, msg));
     }
 
-    // convert to wstring and make WinXP ready
-    static string_t sanitize_url(const std::string& url, int flags)
+    // convert to wstring
+    static string_t sanitize_url(const std::string& url, int /*flags*/)
     {
-        (void)flags;
-    #ifdef _WIN32
-        if (flags & http_client::uses_sni)
-        {
-            // Windows XP doesn't support SNI and so can't connect over SSL to
-            // hosts that use it. The use of SNI is increasingly common and some
-            // APIs Poedit needs to connect to use it. To keep things simple, just
-            // disable SSL on Windows XP.
-            if (is_windows_xp())
-            {
-                if (boost::starts_with(url, "https://"))
-                    return U("http://") + string_t(url.begin() + 8, url.end());
-            }
-        }
-    #endif
         return to_string_t(url);
     }
 
-private:
 #ifdef _WIN32
-    static bool is_windows_xp()
+    // prepare WinHttp configuration
+    static http::client::http_client_config get_client_config()
     {
-        OSVERSIONINFOEX info;
-        ZeroMemory(&info, sizeof(info));
-        info.dwOSVersionInfoSize = sizeof(info);
-        GetVersionEx(reinterpret_cast<OSVERSIONINFO*>(&info));
-        return (info.dwMajorVersion < 6); // XP
+        http::client::http_client_config c;
+
+        // WinHttp doesn't share WinInet/MSIE's proxy settings and has its own,
+        // but many users don't have properly configured both. Adopting proxy
+        // settings like this in desktop software is recommended behavior, see
+        // https ://blogs.msdn.microsoft.com/ieinternals/2013/10/11/understanding-web-proxy-configuration/
+        WINHTTP_CURRENT_USER_IE_PROXY_CONFIG ieConfig = { 0 };
+        if (WinHttpGetIEProxyConfigForCurrentUser(&ieConfig))
+        {
+            if (ieConfig.fAutoDetect)
+            {
+                c.set_proxy(web_proxy::use_auto_discovery);
+            }
+            if (ieConfig.lpszProxy)
+            {
+                // Explicitly add // to the URL to work around a bug in C++ REST SDK's
+                // parsing of proxies with port number in their address
+                // (see https://github.com/Microsoft/cpprestsdk/issues/57)
+                c.set_proxy(uri(L"//" + std::wstring(ieConfig.lpszProxy)));
+            }
+        }
+        return c;
     }
 
     static std::wstring windows_version()
     {
-        OSVERSIONINFOEX info;
-        ZeroMemory(&info, sizeof(info));
+        OSVERSIONINFOEX info = { 0 };
         info.dwOSVersionInfoSize = sizeof(info);
-        GetVersionEx(reinterpret_cast<OSVERSIONINFO*>(&info));
+
+        NTSTATUS(WINAPI *fRtlGetVersion)(LPOSVERSIONINFOEXW);
+        fRtlGetVersion = reinterpret_cast<decltype(fRtlGetVersion)>(GetProcAddress(GetModuleHandleA("ntdll"), "RtlGetVersion"));
+        if (fRtlGetVersion)
+            fRtlGetVersion(&info);
+
         return std::to_wstring(info.dwMajorVersion) + L"." + std::to_wstring(info.dwMinorVersion);
     }
 
@@ -406,18 +338,17 @@ void http_client::set_authorization(const std::string& auth)
     m_impl->set_authorization(auth);
 }
 
-void http_client::get(const std::string& url,
-                      std::function<void(const http_response&)> handler)
+dispatch::future<::json> http_client::get(const std::string& url)
 {
-    m_impl->get(url, handler);
+    return m_impl->get(url);
 }
 
-void http_client::download(const std::string& url, const std::wstring& output_file, response_func_t handler)
+dispatch::future<void> http_client::download(const std::string& url, const std::wstring& output_file)
 {
-    m_impl->download(url, output_file, handler);
+    return m_impl->download(url, output_file);
 }
 
-void http_client::post(const std::string& url, const http_body_data& data, response_func_t handler)
+dispatch::future<::json> http_client::post(const std::string& url, const http_body_data& data)
 {
-    m_impl->post(url, data, handler);
+    return m_impl->post(url, data);
 }
