@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2016 Vaclav Slavik
+ *  Copyright (C) 2013-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -43,7 +43,7 @@ enum class TextDirection
 class Language
 {
 public:
-    Language() {}
+    Language() : m_direction(TextDirection::LTR) {}
 
     bool IsValid() const { return !m_code.empty(); }
     const std::string& Code() const { return m_code; }
