@@ -1,7 +1,7 @@
 ﻿/*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2016 Vaclav Slavik
+ *  Copyright (C) 2014-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -27,8 +27,10 @@
 
 #include "catalog.h"
 #include "customcontrols.h"
+#include "colorscheme.h"
 #include "commentdlg.h"
 #include "concurrency.h"
+#include "configuration.h"
 #include "errors.h"
 #include "hidpi.h"
 #include "utility.h"
@@ -39,7 +41,6 @@
 
 #include <wx/app.h>
 #include <wx/button.h>
-#include <wx/config.h>
 #include <wx/dcclient.h>
 #include <wx/menu.h>
 #include <wx/sizer.h>
@@ -50,7 +51,6 @@
 
 #include <algorithm>
 
-#define SIDEBAR_BACKGROUND      wxColour("#EDF0F4")
 #define GRAY_LINES_COLOR        wxColour(220,220,220)
 #define GRAY_LINES_COLOR_DARK   wxColour(180,180,180)
 
@@ -60,7 +60,7 @@ class SidebarSeparator : public wxWindow
 public:
     SidebarSeparator(wxWindow *parent)
         : wxWindow(parent, wxID_ANY),
-          m_sides(SIDEBAR_BACKGROUND),
+          m_sides(ColorScheme::Get(Color::SidebarBackground)),
           m_center(GRAY_LINES_COLOR_DARK)
     {
         Bind(wxEVT_PAINT, &SidebarSeparator::OnPaint, this);
@@ -71,6 +71,8 @@ public:
         return wxSize(-1, 1);
     }
 
+    bool AcceptsFocus() const override { return false; }
+
 private:
     void OnPaint(wxPaintEvent&)
     {
@@ -133,7 +135,7 @@ public:
         : SidebarBlock(parent, _("Previous source text:"))
     {
         m_innerSizer->AddSpacer(PX(2));
-        m_innerSizer->Add(new ExplanationLabel(parent, _("The old source text (before it changed during an update) that the fuzzy translation corresponds to.")),
+        m_innerSizer->Add(new ExplanationLabel(parent, _("The old source text (before it changed during an update) that the now-inaccurate translation corresponds to.")),
                      wxSizerFlags().Expand());
         m_innerSizer->AddSpacer(PX(5));
         m_text = new SelectableAutoWrappingText(parent, "");
@@ -257,22 +259,30 @@ private:
 
 wxDEFINE_EVENT(EVT_SUGGESTION_SELECTED, wxCommandEvent);
 
-class SuggestionWidget : public wxPanel
+class SuggestionWidget : public wxWindow
 {
 public:
-    SuggestionWidget(wxWindow *parent) : wxPanel(parent, wxID_ANY)
+    SuggestionWidget(wxWindow *parent, bool isFirst) : wxWindow(parent, wxID_ANY)
     {
         m_icon = new wxStaticBitmap(this, wxID_ANY, wxNullBitmap);
         m_text = new AutoWrappingText(this, "TEXT");
         m_info = new InfoStaticText(this);
 
+        m_isPerfect = isFirst
+                      ? new wxStaticBitmap(this, wxID_ANY, wxArtProvider::GetBitmap("SuggestionPerfectMatch"))
+                      : nullptr;
+
         auto top = new wxBoxSizer(wxHORIZONTAL);
         auto right = new wxBoxSizer(wxVERTICAL);
         top->AddSpacer(PX(2));
-        top->Add(m_icon, wxSizerFlags().Top().PXBorder(wxTOP|wxBOTTOM));
+        top->Add(m_icon, wxSizerFlags().Top().Border(wxTOP|wxBOTTOM, PX(6)));
         top->Add(right, wxSizerFlags(1).Expand().PXBorder(wxLEFT));
         right->Add(m_text, wxSizerFlags().Expand().Border(wxTOP, PX(4)));
-        right->Add(m_info, wxSizerFlags().Expand().Border(wxTOP|wxBOTTOM, PX(2)));
+        auto infoSizer = new wxBoxSizer(wxHORIZONTAL);
+        infoSizer->Add(m_info);
+        if (m_isPerfect)
+            infoSizer->Add(m_isPerfect, wxSizerFlags().Center().Border(wxLEFT, PX(2)));
+        right->Add(infoSizer, wxSizerFlags().Expand().Border(wxTOP|wxBOTTOM, PX(2)));
         SetSizerAndFit(top);
 
         // setup mouse hover highlighting:
@@ -292,7 +302,8 @@ public:
     {
         m_value = s;
 
-        auto percent = wxString::Format("%d%%", int(100 * s.score));
+        int percent = int(100 * s.score);
+        auto percentStr = wxString::Format("%d%%", percent);
 
         index++;
         if (index < 10)
@@ -303,22 +314,25 @@ public:
             // TRANSLATORS: This is the key shortcut used in menus on Windows, some languages call them differently
             auto shortcut = wxString::Format("%s%d", _("Ctrl+"), index);
         #endif
-            m_info->SetLabel(wxString::Format(L"%s • %s", shortcut, percent));
+            m_info->SetLabel(wxString::Format(L"%s • %s", shortcut, percentStr));
         }
         else
         {
-            m_info->SetLabel(percent);
+            m_info->SetLabel(percentStr);
         }
 
         m_icon->SetBitmap(icon);
 
+        if (m_isPerfect)
+            m_isPerfect->GetContainingSizer()->Show(m_isPerfect, percent == 100);
+
         auto text = wxControl::EscapeMnemonics(bidi::mark_direction(s.text, lang));
 
         m_text->SetLanguage(lang);
         m_text->SetAndWrapLabel(text);
 
 #ifndef __WXOSX__
-        // FIXME: Causes weird issues on OS X: tooltips appearing on the main list control,
+        // FIXME: Causes weird issues on macOS: tooltips appearing on the main list control,
         //        over toolbar, where the mouse just was etc.
         m_icon->SetToolTip(tooltip);
         m_text->SetToolTip(tooltip);
@@ -330,6 +344,8 @@ public:
         SetMinSize(wxDefaultSize);
         SetMinSize(GetBestSize());
     }
+    
+    bool AcceptsFocus() const override { return false; }
 
 private:
     class InfoStaticText : public wxStaticText
@@ -351,7 +367,7 @@ private:
     void OnMouseMove(wxMouseEvent& e)
     {
         auto rectWin = GetClientRect();
-        rectWin.Deflate(1); // work around off-by-one issue on OS X
+        rectWin.Deflate(1); // work around off-by-one issue on macOS
         auto evtWin = static_cast<wxWindow*>(e.GetEventObject());
         auto mpos = e.GetPosition();
         if (evtWin != this)
@@ -373,6 +389,7 @@ private:
     wxStaticBitmap *m_icon;
     AutoWrappingText *m_text;
     wxStaticText *m_info;
+    wxStaticBitmap *m_isPerfect;
     wxColour m_bg, m_bgHighlight;
 };
 
@@ -406,7 +423,7 @@ SuggestionsSidebarBlock::SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu)
                                      // TRANSLATORS: This is shown when no translation suggestions can be found in the TM (Windows).
                                      _("No matches found")
                                 #else
-                                     // TRANSLATORS: This is shown when no translation suggestions can be found in the TM (OS X, Linux).
+                                     // TRANSLATORS: This is shown when no translation suggestions can be found in the TM (macOS, Linux).
                                      _("No Matches Found")
                                 #endif
                                      );
@@ -427,6 +444,9 @@ SuggestionsSidebarBlock::SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu)
 
 SuggestionsSidebarBlock::~SuggestionsSidebarBlock()
 {
+    ClearSuggestionsMenu();
+    for (auto i : m_suggestionMenuItems)
+        delete i;
 }
 
 wxBitmap SuggestionsSidebarBlock::GetIconForSuggestion(const Suggestion&) const
@@ -456,7 +476,7 @@ void SuggestionsSidebarBlock::SetMessage(const wxString& icon, const wxString& t
     m_parent->Layout();
 }
 
-void SuggestionsSidebarBlock::ReportError(SuggestionsBackend*, std::exception_ptr e)
+void SuggestionsSidebarBlock::ReportError(SuggestionsBackend*, dispatch::exception_ptr e)
 {
     SetMessage("SuggestionError", DescribeException(e));
 }
@@ -484,7 +504,7 @@ void SuggestionsSidebarBlock::UpdateSuggestions(const SuggestionsList& hits)
     // create any necessary controls:
     while (m_suggestions.size() > m_suggestionsWidgets.size())
     {
-        auto w = new SuggestionWidget(m_parent);
+        auto w = new SuggestionWidget(m_parent, /*isFirst=*/m_suggestionsWidgets.empty());
         m_suggestionsSizer->Add(w, wxSizerFlags().Expand());
         m_suggestionsWidgets.push_back(w);
     }
@@ -621,7 +641,7 @@ void SuggestionsSidebarBlock::Show(bool show)
 bool SuggestionsSidebarBlock::ShouldShowForItem(const CatalogItemPtr&) const
 {
     return m_parent->FileHasCapability(Catalog::Cap::Translations) &&
-           wxConfig::Get()->ReadBool("use_tm", true);
+           Config::UseTM();
 }
 
 void SuggestionsSidebarBlock::Update(const CatalogItemPtr& item)
@@ -691,47 +711,40 @@ void SuggestionsSidebarBlock::QueryProvider(SuggestionsBackend& backend, const C
         backend,
         m_parent->GetCurrentSourceLanguage(),
         m_parent->GetCurrentLanguage(),
-        item->GetString().ToStdWstring(),
-
-        // when receiving data
-        [=](const SuggestionsList& hits){
-            call_on_main_thread([weakSelf,queryId,hits]{
-                auto self = weakSelf.lock();
-                // maybe this call is already out of date:
-                if (!self || self->m_latestQueryId != queryId)
-                    return;
-                self->UpdateSuggestions(hits);
-                if (--self->m_pendingQueries == 0)
-                    self->OnQueriesFinished();
-            });
-        },
-
-        // on error:
-        [=](std::exception_ptr e){
-            call_on_main_thread([weakSelf,queryId,backendPtr,e]{
-                auto self = weakSelf.lock();
-                // maybe this call is already out of date:
-                if (!self || self->m_latestQueryId != queryId)
-                    return;
-                self->ReportError(backendPtr, e);
-                if (--self->m_pendingQueries == 0)
-                    self->OnQueriesFinished();
-            });
-        }
-    );
+        item->GetString().ToStdWstring()
+    )
+    .then_on_main([weakSelf,queryId](SuggestionsList hits)
+    {
+        auto self = weakSelf.lock();
+        // maybe this call is already out of date:
+        if (!self || self->m_latestQueryId != queryId)
+            return;
+        self->UpdateSuggestions(hits);
+        if (--self->m_pendingQueries == 0)
+            self->OnQueriesFinished();
+    })
+    .catch_all([weakSelf,queryId,backendPtr](dispatch::exception_ptr e)
+    {
+        auto self = weakSelf.lock();
+        // maybe this call is already out of date:
+        if (!self || self->m_latestQueryId != queryId)
+            return;
+        self->ReportError(backendPtr, e);
+        if (--self->m_pendingQueries == 0)
+            self->OnQueriesFinished();
+    });
 }
 
 
 
 Sidebar::Sidebar(wxWindow *parent, wxMenu *suggestionsMenu)
-    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxNO_BORDER | DoubleBufferingWindowStyle()),
+    : wxWindow(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxNO_BORDER | wxFULL_REPAINT_ON_RESIZE),
       m_catalog(nullptr),
       m_selectedItem(nullptr)
 {
-    SetBackgroundColour(SIDEBAR_BACKGROUND);
+    SetBackgroundColour(ColorScheme::Get(Color::SidebarBackground));
 #ifdef __WXMSW__
-    if (!IsWindowsXP())
-        SetDoubleBuffered(true);
+    SetDoubleBuffered(true);
 #endif
 
     Bind(wxEVT_PAINT, &Sidebar::OnPaint, this);
@@ -847,11 +860,8 @@ void Sidebar::OnPaint(wxPaintEvent&)
 {
     wxPaintDC dc(this);
 
-    dc.SetPen(wxPen(GRAY_LINES_COLOR));
-#ifndef __WXMSW__
-    dc.DrawLine(0, 0, 0, dc.GetSize().y-1);
-#endif
-#ifndef __WXOSX__
+#ifdef __WXOSX__
+    dc.SetPen(ColorScheme::Get(Color::ToolbarSeparator));
     dc.DrawLine(0, 0, dc.GetSize().x - 1, 0);
 #endif
 }
