@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2016 Vaclav Slavik
+ *  Copyright (C) 2013-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -28,23 +28,14 @@
 
 #include <string>
 
+#include <boost/locale/encoding_utf.hpp>
+
 #ifdef __OBJC__
 #include <Foundation/NSString.h>
 #endif
 
-// as of gcc-4.9, no functional <codecvt> support yet
-#if defined(__clang__) || defined(_MSC_VER)
-    #define HAVE_CODECVT
-#endif
-
-#ifdef HAVE_CODECVT
-    #include <locale>
-    #include <codecvt>
-#else
-    #include <boost/locale/encoding_utf.hpp>
-#endif
-
 #include <wx/string.h>
+#include <wx/buffer.h>
 
 /**
     Defines conversions between various string types.
@@ -62,28 +53,6 @@
 namespace str
 {
 
-#ifdef HAVE_CODECVT
-
-inline std::string to_utf8(const std::wstring& str)
-{
-    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> convert;
-    return convert.to_bytes(str.data());
-}
-
-inline std::string to_utf8(const wchar_t *str)
-{
-    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> convert;
-    return convert.to_bytes(str);
-}
-
-inline std::wstring to_wstring(const std::string& utf8str)
-{
-    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> convert;
-    return convert.from_bytes(utf8str.data());
-}
-
-#else
-
 inline std::string to_utf8(const std::wstring& str)
 {
     return boost::locale::conv::utf_to_utf<char>(str);
@@ -99,8 +68,6 @@ inline std::wstring to_wstring(const std::string& utf8str)
     return boost::locale::conv::utf_to_utf<wchar_t>(utf8str);
 }
 
-#endif
-
 inline std::string to_utf8(const wxString& str)
 {
     return std::string(str.utf8_str());
@@ -128,6 +95,11 @@ inline NSString *to_NS(const std::string& utf8str)
     return [NSString stringWithUTF8String:utf8str.c_str()];
 }
 
+inline NSString *to_NS(const char *utf8str)
+{
+    return [NSString stringWithUTF8String:utf8str];
+}
+
 inline std::string to_utf8(NSString *str)
 {
     return std::string([str UTF8String]);
@@ -140,8 +112,7 @@ inline NSString *to_NS(const std::wstring& str)
 
 inline std::wstring to_wstring(NSString *str)
 {
-    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> convert;
-    return convert.from_bytes([str UTF8String]);
+    return boost::locale::conv::utf_to_utf<wchar_t>([str UTF8String]);
 }
 
 #endif // Objective-C++
@@ -188,6 +159,25 @@ inline icu::UnicodeString to_icu(const std::wstring& str)
 #endif
 }
 
+/**
+    Create buffer with raw UChar* string.
+
+    Notice that the resulting string is only valid for the input's lifetime.
+ */
+inline wxScopedCharTypeBuffer<UChar> to_icu_raw(const wxString& str)
+{
+    static_assert(U_SIZEOF_UCHAR == 2, "unexpected UChar size");
+#if SIZEOF_WCHAR_T == 2
+    // read-only aliasing ctor, doesn't copy data
+    return wxScopedCharTypeBuffer<UChar>::CreateNonOwned((const UChar*)str.wx_str(), str.length());
+#else
+    auto buf = wxMBConvUTF16().cWC2MB(str.wc_str());
+    auto len = buf.length();
+    return wxCharTypeBuffer<UChar>::CreateOwned((UChar*)buf.release(), len);
+#endif
+}
+
+
 /// Create wxString from icu::UnicodeString, making a copy.
 inline wxString to_wx(const icu::UnicodeString& str)
 {
