@@ -1,7 +1,7 @@
 ﻿/*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2016 Vaclav Slavik
+ *  Copyright (C) 1999-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -48,6 +48,7 @@
   _COM_SMARTPTR_TYPEDEF(ITextDocument, __uuidof(ITextDocument));
 #endif
 
+#include "colorscheme.h"
 #include "spellchecking.h"
 #include "str_helpers.h"
 #include "unicode_helpers.h"
@@ -202,7 +203,7 @@ private:
 
 #ifdef __WXOSX__
 
-// wxTextCtrl implementation on OS X uses insertText:, which is intended for
+// wxTextCtrl implementation on macOS uses insertText:, which is intended for
 // user input and performs some user input processing, such as autocorrections.
 // We need to avoid this, because Poedit's text control is filled with data
 // when moving in the list control: https://github.com/vslavik/poedit/issues/81
@@ -213,13 +214,12 @@ CustomizedTextCtrl::CustomizedTextCtrl(wxWindow *parent, wxWindowID winid, long
 {
     auto text = TextView(this);
 
-    [text setTextContainerInset:NSMakeSize(1,3)];
+    [text setTextContainerInset:NSMakeSize(0,3)];
     [text setRichText:NO];
 
-    // TODO: This isn't implemented and doesn't work in OS X
-    //Bind(wxEVT_TEXT_COPY, &CustomizedTextCtrl::OnCopy, this);
-    //Bind(wxEVT_TEXT_CUT, &CustomizedTextCtrl::OnCut, this);
-    //Bind(wxEVT_TEXT_PASTE, &CustomizedTextCtrl::OnPaste, this);
+    Bind(wxEVT_TEXT_COPY, &CustomizedTextCtrl::OnCopy, this);
+    Bind(wxEVT_TEXT_CUT, &CustomizedTextCtrl::OnCut, this);
+    Bind(wxEVT_TEXT_PASTE, &CustomizedTextCtrl::OnPaste, this);
 }
 
 void CustomizedTextCtrl::DoSetValue(const wxString& value, int flags)
@@ -259,18 +259,17 @@ wxString CustomizedTextCtrl::DoGetValue() const
 #else // !__WXOSX__
 
 CustomizedTextCtrl::CustomizedTextCtrl(wxWindow *parent, wxWindowID winid, long style)
-   : wxTextCtrl(parent, winid, "", wxDefaultPosition, wxDefaultSize, style | ALWAYS_USED_STYLE)
 {
+    wxTextCtrl::Create(parent, winid, "", wxDefaultPosition, wxDefaultSize, style | ALWAYS_USED_STYLE);
+
     wxTextAttr padding;
     padding.SetLeftIndent(5);
     padding.SetRightIndent(5);
     SetDefaultStyle(padding);
 
-#if defined(__WXMSW__) || defined(__WXGTK__)
     Bind(wxEVT_TEXT_COPY, &CustomizedTextCtrl::OnCopy, this);
     Bind(wxEVT_TEXT_CUT, &CustomizedTextCtrl::OnCut, this);
     Bind(wxEVT_TEXT_PASTE, &CustomizedTextCtrl::OnPaste, this);
-#endif
 
 #ifdef __WXGTK__
     m_historyLocks = 0;
@@ -281,7 +280,21 @@ CustomizedTextCtrl::CustomizedTextCtrl(wxWindow *parent, wxWindowID winid, long
 
 #endif // !__WXOSX__
 
-#if defined(__WXMSW__) || defined(__WXGTK__)
+
+#ifdef __WXMSW__
+
+WXDWORD CustomizedTextCtrl::MSWGetStyle(long style, WXDWORD *exstyle) const
+{
+    auto msStyle = wxTextCtrl::MSWGetStyle(style, exstyle);
+    // Disable always-shown scrollbars. The reason wx does this doesn't seem to
+    // affect Poedit, so it should be safe:
+    msStyle &= ~ES_DISABLENOSCROLL;
+    return msStyle;
+}
+
+#endif // __WXMSW__
+
+
 // We use wxTE_RICH2 style, which allows for pasting rich-formatted
 // text into the control. We want to allow only plain text (all the
 // formatting done is Poedit's syntax highlighting), so we need to
@@ -355,7 +368,6 @@ void CustomizedTextCtrl::DoPasteText(long from, long to, const wxString& s)
 {
     Replace(from, to, s);
 }
-#endif // __WXMSW__/__WXGTK__
 
 #ifdef __WXGTK__
 void CustomizedTextCtrl::BeginUndoGrouping()
@@ -454,17 +466,19 @@ class AnyTranslatableTextCtrl::Attributes
 {
 public:
 #ifdef __WXOSX__
-    NSDictionary *m_attrSpace, *m_attrEscape;
+    NSDictionary *m_attrSpace, *m_attrEscape, *m_attrMarkup, *m_attrFormat;
     typedef NSDictionary* AttrType;
 
     Attributes()
     {
-        m_attrSpace  = @{NSBackgroundColorAttributeName: [NSColor colorWithSRGBRed:0.89 green:0.96 blue:0.68 alpha:1]};
-        m_attrEscape = @{NSBackgroundColorAttributeName: [NSColor colorWithSRGBRed:1 green:0.95 blue:1 alpha:1],
-                         NSForegroundColorAttributeName: [NSColor colorWithSRGBRed:0.46 green:0 blue:0.01 alpha:1]};
+        m_attrSpace  = @{NSBackgroundColorAttributeName: ColorScheme::Get(Color::SyntaxLeadingWhitespaceBg).OSXGetNSColor()};
+        m_attrEscape = @{NSBackgroundColorAttributeName: ColorScheme::Get(Color::SyntaxEscapeBg).OSXGetNSColor(),
+                         NSForegroundColorAttributeName: ColorScheme::Get(Color::SyntaxEscapeFg).OSXGetNSColor()};
+        m_attrMarkup = @{NSForegroundColorAttributeName: ColorScheme::Get(Color::SyntaxMarkup).OSXGetNSColor()};
+        m_attrFormat = @{NSForegroundColorAttributeName: ColorScheme::Get(Color::SyntaxFormat).OSXGetNSColor()};
     }
 #else // !__WXOSX__
-    wxTextAttr m_attrDefault, m_attrSpace, m_attrEscape;
+    wxTextAttr m_attrDefault, m_attrSpace, m_attrEscape, m_attrMarkup, m_attrFormat;
     typedef wxTextAttr AttrType;
 
     Attributes()
@@ -472,10 +486,14 @@ public:
         m_attrDefault.SetBackgroundColour(*wxWHITE);
         m_attrDefault.SetTextColour(*wxBLACK);
 
-        m_attrSpace.SetBackgroundColour("#E4F6AE");
+        m_attrSpace.SetBackgroundColour(ColorScheme::Get(Color::SyntaxLeadingWhitespaceBg));
 
-        m_attrEscape.SetBackgroundColour("#FFF1FF");
-        m_attrEscape.SetTextColour("#760003");
+        m_attrEscape.SetBackgroundColour(ColorScheme::Get(Color::SyntaxEscapeBg));
+        m_attrEscape.SetTextColour(ColorScheme::Get(Color::SyntaxEscapeFg));
+
+        m_attrMarkup.SetTextColour(ColorScheme::Get(Color::SyntaxMarkup));
+
+        m_attrFormat.SetTextColour(ColorScheme::Get(Color::SyntaxFormat));
     }
 
     const AttrType& Default() const {  return m_attrDefault; }
@@ -487,6 +505,8 @@ public:
         {
             case SyntaxHighlighter::LeadingWhitespace:  return m_attrSpace;
             case SyntaxHighlighter::Escape:             return m_attrEscape;
+            case SyntaxHighlighter::Markup:             return m_attrMarkup;
+            case SyntaxHighlighter::Format:             return m_attrFormat;
         }
         return m_attrSpace; // silence bogus warning
     }
@@ -513,10 +533,25 @@ void AnyTranslatableTextCtrl::SetLanguage(const Language& lang)
 {
     m_language = lang;
 
+    wxEventBlocker block(this, wxEVT_TEXT);
+
 #ifdef __WXOSX__
     NSTextView *text = TextView(this);
-    [text setBaseWritingDirection:lang.IsRTL() ? NSWritingDirectionRightToLeft : NSWritingDirectionLeftToRight];
+    if (lang.IsRTL())
+    {
+        [text setBaseWritingDirection:NSWritingDirectionRightToLeft];
+        if ([NSApp userInterfaceLayoutDirection] == NSUserInterfaceLayoutDirectionLeftToRight)
+        {
+            // extra nudge to make typing behave as expected:
+            [text makeTextWritingDirectionRightToLeft:nil];
+        }
+    }
+    else
+    {
+        [text setBaseWritingDirection:NSWritingDirectionLeftToRight];
+    }
 #endif
+
 #ifdef __WXMSW__
     BIDIOPTIONS bidi;
     ::ZeroMemory(&bidi, sizeof(bidi));
@@ -546,7 +581,7 @@ wxString AnyTranslatableTextCtrl::GetPlainText() const
 wxString AnyTranslatableTextCtrl::EscapePlainText(const wxString& s)
 {
     // Note: the escapes used here should match with
-    //       SyntaxHighlighter::Highlight() ones
+    //       BasicSyntaxHighlighter::Highlight() ones
     wxString s2;
     s2.reserve(s.length());
     for (auto i = s.begin(); i != s.end(); ++i)
@@ -653,7 +688,6 @@ wxString AnyTranslatableTextCtrl::UnescapePlainText(const wxString& s)
     return s2;
 }
 
-#if defined(__WXMSW__) || defined(__WXGTK__)
 wxString AnyTranslatableTextCtrl::DoCopyText(long from, long to)
 {
     return UnescapePlainText(GetRange(from, to));
@@ -663,16 +697,20 @@ void AnyTranslatableTextCtrl::DoPasteText(long from, long to, const wxString& s)
 {
     Replace(from, to, EscapePlainText(s));
 }
-#endif
 
-#ifdef __WXMSW__
 void AnyTranslatableTextCtrl::DoSetValue(const wxString& value, int flags)
 {
+#ifdef __WXMSW__
     wxWindowUpdateLocker dis(this);
+#endif
     CustomizedTextCtrl::DoSetValue(value, flags);
+#ifdef __WXMSW__
     UpdateRTLStyle();
+#endif
+    HighlightText();
 }
 
+#ifdef __WXMSW__
 void AnyTranslatableTextCtrl::UpdateRTLStyle()
 {
     wxEventBlocker block(this, wxEVT_TEXT);
@@ -691,15 +729,7 @@ void AnyTranslatableTextCtrl::UpdateRTLStyle()
     ::SendMessage((HWND) GetHWND(), EM_SETPARAFORMAT, 0, (LPARAM) &pf);
     SetSelection(start, end);
 }
-#endif // __WXMSW__
-
-#ifdef __WXGTK__
-void AnyTranslatableTextCtrl::DoSetValue(const wxString& value, int flags)
-{
-    CustomizedTextCtrl::DoSetValue(value, flags);
-    HighlightText();
-}
-#endif
+#endif // !__WXMSW__
 
 void AnyTranslatableTextCtrl::HighlightText()
 {
@@ -712,9 +742,12 @@ void AnyTranslatableTextCtrl::HighlightText()
     [layout removeTemporaryAttribute:NSForegroundColorAttributeName forCharacterRange:fullRange];
     [layout removeTemporaryAttribute:NSBackgroundColorAttributeName forCharacterRange:fullRange];
 
-    m_syntax.Highlight(text, [=](int a, int b, SyntaxHighlighter::TextKind kind){
-        [layout addTemporaryAttributes:m_attrs->For(kind) forCharacterRange:NSMakeRange(a, b-a)];
-    });
+    if (m_syntax)
+    {
+        m_syntax->Highlight(text, [=](int a, int b, SyntaxHighlighter::TextKind kind){
+            [layout addTemporaryAttributes:m_attrs->For(kind) forCharacterRange:NSMakeRange(a, b-a)];
+        });
+    }
 
 #else // !__WXOSX__
 
@@ -734,24 +767,26 @@ void AnyTranslatableTextCtrl::HighlightText()
     deflt.SetFont(GetFont());
     SetStyle(0, text.length(), deflt);
 
-    m_syntax.Highlight(text, [=](int a, int b, SyntaxHighlighter::TextKind kind){
-        SetStyle(a, b, m_attrs->For(kind));
-    });
-
+    if (m_syntax)
+    {
+        m_syntax->Highlight(text, [=](int a, int b, SyntaxHighlighter::TextKind kind){
+            SetStyle(a, b, m_attrs->For(kind));
+        });
+    }
 #endif // __WXOSX__/!__WXOSX__
 }
 
 
 
 SourceTextCtrl::SourceTextCtrl(wxWindow *parent, wxWindowID winid)
-    : AnyTranslatableTextCtrl(parent, winid, wxTE_READONLY)
+    : AnyTranslatableTextCtrl(parent, winid, wxTE_READONLY | wxNO_BORDER)
 {
     SetLanguage(Language::English());
 }
 
 
 TranslationTextCtrl::TranslationTextCtrl(wxWindow *parent, wxWindowID winid)
-    : AnyTranslatableTextCtrl(parent, winid),
+    : AnyTranslatableTextCtrl(parent, winid, wxNO_BORDER),
       m_lastKeyWasReturn(false)
 {
 #ifdef __WXMSW__
@@ -818,4 +853,6 @@ void TranslationTextCtrl::SetPlainTextUserWritten(const wxString& value)
     SelectAll();
     WriteText(EscapePlainText(value));
     SetInsertionPointEnd();
+
+    HighlightText();
 }
