@@ -1,7 +1,7 @@
 ﻿/*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2016 Vaclav Slavik
+ *  Copyright (C) 1999-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -30,13 +30,14 @@
 #include <memory>
 #include <vector>
 
+#include "language.h"
 #include "syntaxhighlighter.h"
 
 
 /**
     Text control with some Poedit-specific customizations:
     
-    - Allow setting text programatically, without user-input processing (OS X)
+    - Allow setting text programatically, without user-input processing (macOS)
     - Disable user-usable rich text support
     - Stylistic tweaks (padding and such)
     - Generic undo/redo implementation for GTK
@@ -45,7 +46,7 @@
 class CustomizedTextCtrl : public wxTextCtrl
 {
 public:
-    static const int ALWAYS_USED_STYLE = wxTE_MULTILINE | wxTE_RICH2;
+    static const int ALWAYS_USED_STYLE = wxTE_MULTILINE | wxTE_RICH2 | wxTE_NOHIDESEL;
 
     CustomizedTextCtrl(wxWindow *parent, wxWindowID winid, long style = 0);
 
@@ -59,13 +60,16 @@ public:
     void SaveSnapshot();
 #endif
 
+#ifdef __WXMSW__
+    WXDWORD MSWGetStyle(long style, WXDWORD *exstyle) const override;
+#endif
+
 protected:
 #ifdef __WXOSX__
     void DoSetValue(const wxString& value, int flags) override;
     wxString DoGetValue() const override;
 #endif
 
-#if defined(__WXMSW__) || defined(__WXGTK__)
     bool DoCopy();
     void OnCopy(wxClipboardTextEvent& event);
     void OnCut(wxClipboardTextEvent& event);
@@ -73,7 +77,6 @@ protected:
 
     virtual wxString DoCopyText(long from, long to);
     virtual void DoPasteText(long from, long to, const wxString& s);
-#endif // __WXMSW__/__WXGTK__
 
 #ifdef __WXGTK__
     struct Snapshot
@@ -102,42 +105,37 @@ public:
     AnyTranslatableTextCtrl(wxWindow *parent, wxWindowID winid, int style = 0);
     ~AnyTranslatableTextCtrl();
 
-    void SetLanguageRTL(bool isRTL);
+    void SetLanguage(const Language& lang);
+    void SetSyntaxHighlighter(SyntaxHighlighterPtr syntax) { m_syntax = syntax; }
 
     // Set and get control's text as plain/raw text, with no escaping or formatting.
     // This is the "true" representation, with e.g newlines included. The version
     // displayed to the user includes syntax highlighting and escaping of some characters
     // (e.g. tabs shown as \t, newlines as \n followed by newline).
-    void SetPlainText(const wxString& s) { SetValue(EscapePlainText(s)); }
-    wxString GetPlainText() const { return UnescapePlainText(GetValue()); }
+    void SetPlainText(const wxString& s);
+    wxString GetPlainText() const;
 
     // Apply escaping as described in SetPlainText:
     static wxString EscapePlainText(const wxString& s);
     static wxString UnescapePlainText(const wxString& s);
 
 protected:
-#if defined(__WXMSW__) || defined(__WXGTK__)
     wxString DoCopyText(long from, long to) override;
     void DoPasteText(long from, long to, const wxString& s) override;
-#endif
+
+    void DoSetValue(const wxString& value, int flags) override;
 
 #ifdef __WXMSW__
-    void DoSetValue(const wxString& value, int flags) override;
     void UpdateRTLStyle();
-
-    bool m_isRTL;
 #endif // __WXMSW__
 
-#ifdef __WXGTK__
-    void DoSetValue(const wxString& value, int flags) override;
-#endif
-
 protected:
     void HighlightText();
 
     class Attributes;
-    SyntaxHighlighter m_syntax;
+    SyntaxHighlighterPtr m_syntax;
     std::unique_ptr<Attributes> m_attrs;
+    Language m_language;
 };
 
 
