@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2010-2016 Vaclav Slavik
+ *  Copyright (C) 2010-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -32,6 +32,8 @@
     #endif
 #endif
 
+#include <map>
+
 #include <wx/arrstr.h>
 #include <wx/filename.h>
 #include <wx/string.h>
@@ -52,17 +54,17 @@
 #endif
 
 #ifdef __WXOSX__
-    #define OSX_OR_OTHER(msw, other) msw
+    #define MACOS_OR_OTHER(mac, other) mac
 #else
-    #define OSX_OR_OTHER(msw, other) other
+    #define MACOS_OR_OTHER(mac, other) other
 #endif
 
 #ifdef __WXOSX__
     #define BORDER_WIN(dir, n) Border(dir, 0)
-    #define BORDER_OSX(dir, n) Border(dir, n)
+    #define BORDER_MACOS(dir, n) Border(dir, n)
 #else
     #define BORDER_WIN(dir, n) Border(dir, n)
-    #define BORDER_OSX(dir, n) Border(dir, 0)
+    #define BORDER_MACOS(dir, n) Border(dir, 0)
 #endif
 
 
@@ -78,31 +80,8 @@ inline bool IsWindows10OrGreater()
     return wxGetOsVersion(&osmajor, nullptr) == wxOS_WINDOWS_NT && osmajor >= 10;
 }
 
-inline bool IsWindowsXP()
-{
-    int osmajor = 0;
-    wxGetOsVersion(&osmajor, nullptr);
-    return osmajor < 6;
-}
-
-inline bool IsWindows7OrGreater()
-{
-    int verMaj, verMin;
-    wxGetOsVersion(&verMaj, &verMin);
-    return verMaj > 6 || (verMaj == 6 && verMin >= 1);
-}
-
 #endif // __WXMSW__
 
-inline int DoubleBufferingWindowStyle()
-{
-#ifdef __WXMSW__
-    return IsWindowsXP() ? 0 : wxFULL_REPAINT_ON_RESIZE;
-#else
-    return 0;
-#endif
-}
-
 // ----------------------------------------------------------------------
 // Misc helpers
 // ----------------------------------------------------------------------
@@ -231,7 +210,7 @@ public:
     static void KeepFiles(bool keep = true) { ms_keepFiles = keep; }
 
 private:
-    int m_counter;
+    std::map<wxString, int> m_counters;
     wxString m_dir;
 
     static bool ms_keepFiles;
