@@ -1,7 +1,7 @@
 ﻿/*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2016 Vaclav Slavik
+ *  Copyright (C) 2013-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,7 @@
 #include "edframe.h"
 #include "hidpi.h"
 #include "str_helpers.h"
+#include "utility.h"
 
 #include <wx/dcbuffer.h>
 #include <wx/statbmp.h>
@@ -200,43 +201,54 @@ WelcomeScreenPanel::WelcomeScreenPanel(wxWindow *parent)
     uberSizer->AddStretchSpacer();
     SetSizer(uberSizer);
 
+    auto headerSizer = new wxBoxSizer(wxVERTICAL);
+
     auto hdr = new wxStaticBitmap(this, wxID_ANY, wxArtProvider::GetBitmap("PoeditWelcome"));
-    sizer->Add(hdr, wxSizerFlags().Center());
+    headerSizer->Add(hdr, wxSizerFlags().Center());
 
     auto header = new HeaderStaticText(this, wxID_ANY, _("Welcome to Poedit"));
     header->SetFont(m_fntHeader);
     header->SetForegroundColour(m_clrHeader);
-    sizer->Add(header, wxSizerFlags().Center().Border(wxTOP, PX(10)));
+    headerSizer->Add(header, wxSizerFlags().Center().Border(wxTOP, PX(10)));
 
     auto version = new wxStaticText(this, wxID_ANY, wxString::Format(_("Version %s"), wxGetApp().GetAppVersion()));
     version->SetFont(m_fntSub);
     version->SetForegroundColour(m_clrSub);
-    sizer->Add(version, wxSizerFlags().Center());
+    headerSizer->Add(version, wxSizerFlags().Center());
 
-    sizer->AddSpacer(PX(20));
+    headerSizer->AddSpacer(PX(20));
+
+    sizer->Add(headerSizer, wxSizerFlags().Expand());
 
     sizer->Add(new ActionButton(
                        this, wxID_OPEN,
-                       _("Edit a translation"),
+                       MSW_OR_OTHER(_("Edit a translation"), _("Edit a Translation")),
                        _("Open an existing PO file and edit the translation.")),
                wxSizerFlags().PXBorderAll().Expand());
 
     sizer->Add(new ActionButton(
                        this, XRCID("menu_new_from_pot"),
-                       _("Create new translation"),
+                       MSW_OR_OTHER(_("Create new translation"), _("Create New Translation")),
                        _("Take an existing PO file or POT template and create a new translation from it.")),
                wxSizerFlags().PXBorderAll().Expand());
 
 #ifdef HAVE_HTTP_CLIENT
     sizer->Add(new ActionButton(
                        this, XRCID("menu_open_crowdin"),
-                       _("Collaborate on a translation with Crowdin"),
+                       MSW_OR_OTHER(_("Collaborate on a translation with others"), _("Collaborate on a Translation with Others")),
                        _("Download a file from Crowdin project, translate and sync your changes back.")),
                wxSizerFlags().PXBorderAll().Expand());
     sizer->Add(new LearnAboutCrowdinLink(this, _("What is Crowdin?")), wxSizerFlags().Right().Border(wxRIGHT, PX(8)));
 #endif // HAVE_HTTP_CLIENT
 
     sizer->AddSpacer(PX(50));
+
+    // Hide the cosmetic logo part if the screen is too small:
+    auto minFullSize = sizer->GetMinSize().y + PX(50);
+    Bind(wxEVT_SIZE, [=](wxSizeEvent& e){
+        sizer->Show((size_t)0, e.GetSize().y >= minFullSize);
+        e.Skip();
+    });
 }
 
 
