@@ -1,7 +1,7 @@
 ﻿/*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2016 Vaclav Slavik
+ *  Copyright (C) 2014-2017 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -25,10 +25,19 @@
 
 #include "main_toolbar.h"
 
+#include "hidpi.h"
+#include "utility.h"
+#include "unicode_helpers.h"
+
 #include <wx/intl.h>
+#include <wx/settings.h>
 #include <wx/toolbar.h>
 #include <wx/xrc/xmlres.h>
 
+#ifdef __WXMSW__
+#include <wx/msw/uxtheme.h>
+#endif
+
 
 class WXMainToolbar : public MainToolbar
 {
@@ -36,29 +45,26 @@ public:
     WXMainToolbar(wxFrame *parent)
     {
         m_tb = wxXmlResource::Get()->LoadToolBar(parent, "toolbar");
-        m_idFuzzy = XRCID("menu_fuzzy");
         m_idUpdate = XRCID("toolbar_update");
 
 #ifdef __WXMSW__
         // De-uglify the toolbar a bit on Windows 10:
-        int osmajor = 0;
-        if (wxGetOsVersion(&osmajor, nullptr) == wxOS_WINDOWS_NT && osmajor == 10)
+        if (IsWindows10OrGreater())
         {
-            m_tb->SetBackgroundColour(*wxWHITE);
+            const wxUxThemeEngine* theme = wxUxThemeEngine::GetIfActive();
+            if (theme)
+            {
+                wxUxThemeHandle hTheme(m_tb, L"ExplorerMenu::Toolbar");
+                m_tb->SetBackgroundColour(wxRGBToColour(theme->GetThemeSysColor(hTheme, COLOR_WINDOW)));
+            }
+
+            unsigned padding = PX(4);
+            ::SendMessage((HWND) m_tb->GetHWND(), TB_SETPADDING, 0, MAKELPARAM(padding, padding));
         }
+        m_tb->SetDoubleBuffered(true);
 #endif
     }
 
-    bool IsFuzzy() const override
-    {
-        return m_tb->GetToolState(m_idFuzzy);
-    }
-
-    void SetFuzzy(bool on) override
-    {
-        m_tb->ToggleTool(m_idFuzzy, on);
-    }
-
     void EnableSyncWithCrowdin(bool on) override
     {
         auto tool = m_tb->FindById(m_idUpdate);
@@ -67,19 +73,25 @@ public:
         {
             tool->SetLabel(_("Sync"));
             tool->SetShortHelp(_("Synchronize the translation with Crowdin"));
-            m_tb->SetToolNormalBitmap(m_idUpdate, wxArtProvider::GetBitmap("poedit-sync"));
+            m_tb->SetToolNormalBitmap(m_idUpdate, wxArtProvider::GetBitmap("poedit-sync", wxART_TOOLBAR));
+            #ifdef __WXMSW__
+            m_tb->SetToolDisabledBitmap(m_idUpdate, wxArtProvider::GetBitmap("poedit-sync@disabled", wxART_TOOLBAR));
+            #endif
         }
         else
         {
             tool->SetLabel(_("Update"));
             tool->SetShortHelp(_("Update catalog - synchronize it with sources"));
-            m_tb->SetToolNormalBitmap(m_idUpdate, wxArtProvider::GetBitmap("poedit-update"));
+            m_tb->SetToolNormalBitmap(m_idUpdate, wxArtProvider::GetBitmap("poedit-update", wxART_TOOLBAR));
+            #ifdef __WXMSW__
+            m_tb->SetToolDisabledBitmap(m_idUpdate, wxArtProvider::GetBitmap("poedit-update@disabled", wxART_TOOLBAR));
+            #endif
         }
     }
 
 private:
     wxToolBar *m_tb;
-    int m_idFuzzy, m_idUpdate;
+    int m_idUpdate;
 };
 
 
